@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## system-config-printer
 
-## Copyright (C) 2008, 2011 Red Hat, Inc.
+## Copyright (C) 2008, 2011, 2014 Red Hat, Inc.
 ## Copyright (C) 2008 Till Kamppeter <till.kamppeter@gmail.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -19,10 +19,11 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import pycurl, urllib, platform, threading, tempfile, traceback
+import requests, urllib.request, urllib.parse, urllib.error, platform, threading, tempfile, traceback
 import os, sys
 from xml.etree.ElementTree import XML
 from . import Device
+from . import _debugprint
 
 __all__ = ['OpenPrinting']
 
@@ -42,47 +43,41 @@ class _QueryThread (threading.Thread):
         self.parameters = parameters
         self.callback = callback
         self.user_data = user_data
-        self.result = ""
+        self.result = b''
 
         self.setDaemon (True)
+        _debugprint ("+%s" % self)
+
+    def __del__ (self):
+        _debugprint ("-%s" % self)
 
     def run (self):
 
-        # Callback function for pycURL collecting the data coming from
-        # the web server
-        def collect_data(result):
-            self.result += result;
-            return len(result)
-
         # CGI script to be executed
         query_command = "/query.cgi"
         # Headers for the post request
         headers = {"Content-type": "application/x-www-form-urlencoded",
                    "Accept": "text/plain"}
         params = ("%s&uilanguage=%s&locale=%s" %
-                  (urllib.urlencode (self.parameters),
+                  (urllib.parse.urlencode (self.parameters),
                    self.parent.language[0],
                    self.parent.language[0]))
         self.url = "https://%s%s?%s" % (self.parent.base_url, query_command, params)
         # Send request
         result = None
-        self.result = ""
+        self.result = b''
         status = 1
         try:
-            curl = pycurl.Curl()
-            curl.setopt(pycurl.SSL_VERIFYPEER, 1)
-            curl.setopt(pycurl.SSL_VERIFYHOST, 2)
-            curl.setopt(pycurl.WRITEFUNCTION, collect_data)
-            curl.setopt(pycurl.URL, self.url)
-            status = curl.perform()
-            if status == None: status = 0
-            if (status != 0):
-                self.result = sys.exc_info ()
+            req = requests.get(self.url, verify=True)
+            self.result = req.content
+            status = 0
         except:
             self.result = sys.exc_info ()
-            if status == None: status = 0
+            if status is None:
+                status = 0
 
-        if self.callback != None:
+        _debugprint ("%s: query complete" % self)
+        if self.callback is not None:
             self.callback (status, self.user_data, self.result)
 
 class OpenPrinting:
@@ -92,11 +87,11 @@ class OpenPrinting:
         locale.setlocale().
         @type language: string
         """
-        if language == None:
+        if language is None:
             import locale
             try:
                 language = locale.getlocale(locale.LC_MESSAGES)
-            except locale.Error, e:
+            except locale.Error:
                 language = 'C'
         self.language = language
 
@@ -107,6 +102,11 @@ class OpenPrinting:
         # config file
         self.onlyfree = 1
         self.onlymanufacturer = 0
+        _debugprint ("OpenPrinting: Init %s %s %s" % (self.language, self.onlyfree, self.onlymanufacturer))
+        _debugprint ("+%s" % self)
+
+    def __del__ (self):
+        _debugprint ("-%s" % self)
 
     def cancelOperation(self, handle):
         """
@@ -166,7 +166,7 @@ class OpenPrinting:
                     id = printer.find ("id")
                     make = printer.find ("make")
                     model = printer.find ("model")
-                    if id != None and make != None and model != None:
+                    if id is not None and make is not None and model is not None:
                         idtxt = id.text
                         maketxt = make.text
                         modeltxt = model.text
@@ -176,6 +176,7 @@ class OpenPrinting:
                 status = 1
                 printers = sys.exc_info ()
 
+            _debugprint ("searchPrinters/parse_result: OpenPrinting entries: %s" % repr(printers))
             try:
                 callback (status, user_data, printers)
             except:
@@ -192,6 +193,7 @@ class OpenPrinting:
         params = { 'type': 'printers',
                    'printer': searchterm,
                    'format': 'xml' }
+        _debugprint ("searchPrinters: Querying OpenPrinting: %s" % repr(params))
         return self.webQuery(params, parse_result, (callback, user_data))
 
     def listDrivers(self, model, callback, user_data=None, extra_options=None):
@@ -250,6 +252,7 @@ class OpenPrinting:
                 #       { arch:
                 #         { file:
                 #           { 'url': url,
+                #             'fingerprint': signature key fingerprint URL
                 #             'realversion': upstream version string,
                 #             'version': packaged version string,
                 #             'release': package release string
@@ -264,24 +267,37 @@ class OpenPrinting:
 
                 for driver in root.findall ('driver'):
                     id = driver.attrib.get ('id')
-                    if id == None:
+                    if id is None:
                         continue
 
                     dict = {}
                     for attribute in ['name', 'url', 'supplier', 'license',
                                       'shortdescription' ]:
                         element = driver.find (attribute)
-                        if element != None and element.text != None:
+                        if element is not None and element.text is not None:
                             dict[attribute] = _normalize_space (element.text)
 
                     element = driver.find ('licensetext')
-                    if element != None:
+                    if element is not None and element.text is not None:
                         dict['licensetext'] = element.text
+                    if not 'licensetext' in dict or \
+                       dict['licensetext'] is None:
+                        element = driver.find ('licenselink')
+                        if element is not None:
+                            license_url = element.text
+                            if license_url is not None:
+                                try:
+                                    req = requests.get(license_url, verify=True)
+                                    dict['licensetext'] = \
+                                        req.content.decode("utf-8")
+                                except:
+                                    _debugprint('Cannot retrieve %s' %
+                                                license_url)
 
                     for boolean in ['nonfreesoftware', 'recommended',
                                     'patents', 'thirdpartysupplied',
                                     'manufacturersupplied']:
-                        dict[boolean] = driver.find (boolean) != None
+                        dict[boolean] = driver.find (boolean) is not None
 
                     # Make a 'freesoftware' tag for compatibility with
                     # how the OpenPrinting API used to work (see trac
@@ -290,10 +306,10 @@ class OpenPrinting:
 
                     supportcontacts = []
                     container = driver.find ('supportcontacts')
-                    if container != None:
+                    if container is not None:
                         for sc in container.findall ('supportcontact'):
                             supportcontact = {}
-                            if sc.text != None:
+                            if sc.text is not None:
                                 supportcontact['name'] = \
                                     _normalize_space (sc.text)
                             else:
@@ -305,35 +321,36 @@ class OpenPrinting:
                     if supportcontacts:
                         dict['supportcontacts'] = supportcontacts
 
-                    if not dict.has_key ('name') or not dict.has_key ('url'):
+                    if 'name' not in dict or 'url' not in dict:
                         continue
 
                     container = driver.find ('functionality')
-                    if container != None:
+                    if container is not None:
                         functionality = {}
                         for attribute in ['text', 'lineart', 'graphics',
                                           'photo', 'speed']:
                             element = container.find (attribute)
-                            if element != None:
+                            if element is not None:
                                 functionality[attribute] = element.text
                         if functionality:
                             dict[container.tag] = functionality
 
                     packages = {}
                     container = driver.find ('packages')
-                    if container != None:
+                    if container is not None:
                         for arch in container.getchildren ():
                             rpms = {}
                             for package in arch.findall ('package'):
                                 rpm = {}
                                 for attribute in ['realversion','version',
-                                                  'release', 'url', 'pkgsys']:
+                                                  'release', 'url', 'pkgsys',
+                                                  'fingerprint']:
                                     element = package.find (attribute)
-                                    if element != None:
+                                    if element is not None:
                                         rpm[attribute] = element.text
 
                                 repositories = package.find ('repositories')
-                                if repositories != None:
+                                if repositories is not None:
                                     for pkgsys in repositories.getchildren ():
                                         rpm.setdefault('repositories', {})[pkgsys.tag] = pkgsys.text
 
@@ -345,7 +362,7 @@ class OpenPrinting:
 
                     ppds = []
                     container = driver.find ('ppds')
-                    if container != None:
+                    if container is not None:
                         for each in container.getchildren ():
                             ppds.append (each.text)
 
@@ -353,6 +370,7 @@ class OpenPrinting:
                         dict['ppds'] = ppds
 
                     drivers[id] = dict
+                    _debugprint ("listDrivers/parse_result: OpenPrinting entries: %s" % repr(drivers))
                 callback (0, user_data, drivers)
             except:
                 callback (1, user_data, sys.exc_info ())
@@ -360,11 +378,19 @@ class OpenPrinting:
         if isinstance(model, Device):
             model = model.id
 
+        architecture = platform.machine()
+
+        # On Intel, we could be running a 32bit user space with a 64bit kernel, in
+        # which case platform.machine() will return x86_64, leading to downloading
+        # the wrong printer driver, so we make sure we ask for i386 in that case.
+        if architecture == 'x86_64' and platform.architecture()[0] == '32bit':
+            architecture = 'i386'
+
         params = { 'type': 'drivers',
                    'moreinfo': '1',
                    'showprinterid': '1',
                    'onlynewestdriverpackages': '1',
-                   'architectures': platform.machine(),
+                   'architectures': architecture,
                    'noobsoletes': '1',
                    'onlyfree': str (self.onlyfree),
                    'onlymanufacturer': str (self.onlymanufacturer),
@@ -372,38 +398,40 @@ class OpenPrinting:
                    'format': 'xml'}
         if extra_options:
             params.update(extra_options)
+        _debugprint ("listDrivers: Querying OpenPrinting: %s" % repr(params))
         return self.webQuery(params, parse_result, (callback, user_data))
 
 def _simple_gui ():
-    import gtk, pprint
-    gtk.gdk.threads_init ()
+    from gi.repository import Gdk
+    from gi.repository import Gtk
+    import pprint
+    Gdk.threads_init ()
     class QueryApp:
         def __init__(self):
             self.openprinting = OpenPrinting()
-            self.main = gtk.Dialog ("OpenPrinting query application",
-                                    None,
-                                    gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR,
-                                    (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE,
-                                     "Search", 10,
-                                     "List", 20))
+            self.main = Gtk.Dialog (title="OpenPrinting query application",
+                                    transient_for=None, modal=True)
+            self.main.add_buttons (Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE,
+                                   "Search", 10,
+                                   "List", 20)
             self.main.set_border_width (6)
             self.main.vbox.set_spacing (2)
-            vbox = gtk.VBox (False, 6)
+            vbox = Gtk.VBox.new (False, 6)
             self.main.vbox.pack_start (vbox, True, True, 0)
             vbox.set_border_width (6)
-            self.entry = gtk.Entry ()
+            self.entry = Gtk.Entry ()
             vbox.pack_start (self.entry, False, False, 6)
-            sw = gtk.ScrolledWindow ()
-            self.tv = gtk.TextView ()
+            sw = Gtk.ScrolledWindow ()
+            self.tv = Gtk.TextView ()
             sw.add (self.tv)
             vbox.pack_start (sw, True, True, 6)
             self.main.connect ("response", self.response)
             self.main.show_all ()
 
         def response (self, dialog, response):
-            if (response == gtk.RESPONSE_CLOSE or
-                response == gtk.RESPONSE_DELETE_EVENT):
-                gtk.main_quit ()
+            if (response == Gtk.ResponseType.CLOSE or
+                response == Gtk.ResponseType.DELETE_EVENT):
+                Gtk.main_quit ()
 
             if response == 10:
                 # Run a query.
@@ -421,24 +449,24 @@ def _simple_gui ():
             text = ""
             for printer in printers.values ():
                 text += printer + "\n"
-            gtk.gdk.threads_enter ()
+            Gdk.threads_enter ()
             self.tv.get_buffer ().set_text (text)
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
 
         def list_drivers_callback (self, status, user_data, drivers):
             if status != 0:
                 raise drivers[1]
 
             text = pprint.pformat (drivers)
-            gtk.gdk.threads_enter ()
+            Gdk.threads_enter ()
             self.tv.get_buffer ().set_text (text)
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
 
         def query_callback (self, status, user_data, result):
-            gtk.gdk.threads_enter ()
+            Gdk.threads_enter ()
             self.tv.get_buffer ().set_text (str (result))
-            file ("result.xml", "w").write (str (result))
-            gtk.gdk.threads_leave ()
+            open ("result.xml", "w").write (str (result))
+            Gdk.threads_leave ()
 
     q = QueryApp()
-    gtk.main ()
+    Gtk.main ()
