@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
+from functools import reduce
 cups.require ("1.9.42")
 import sys
 from debug import *
@@ -25,33 +26,29 @@ from debug import *
 import dbus
 import dbus.glib
 import dbus.service
-import gobject
+from gi.repository import GObject
+from gi.repository import GLib
 import time
 import locale
-import gettext
 import cupshelpers.installdriver
-from gettext import gettext as _
+
 DOMAIN="system-config-printer"
-gettext.textdomain (DOMAIN)
+import gettext
+gettext.install(domain=DOMAIN)
 try:
     locale.setlocale (locale.LC_ALL, "")
-except locale.Error, e:
+except locale.Error as e:
     import os
     os.environ['LC_ALL'] = 'C'
     locale.setlocale (locale.LC_ALL, "")
 
-try:
-    import pynotify
-except RuntimeError, e:
-    print "%s:" % DOMAIN, e
-    print "This is a graphical application and requires DISPLAY to be set."
-    sys.exit (1)
+from gi.repository import Notify
 
-APPDIR="/usr/local/share/system-config-printer"
+APPDIR="/usr/share/system-config-printer"
 ICON="printer"
 
-# We need to call pynotify.init before we can check the server for caps
-pynotify.init('System Config Printer Notification')
+# We need to call Notify.init before we can check the server for caps
+Notify.init('System Config Printer Notification')
 
 # D-Bus APIs of other objects we'll use.
 PRINTING_BUS="org.fedoraproject.Config.Printing"
@@ -84,27 +81,27 @@ class NewPrinterNotification(dbus.service.Object):
     def GetReady (self):
         TIMEOUT=1200000
         if self.getting_ready == 0:
-            n = pynotify.Notification (_("Configuring new printer"),
-                                       _("Please wait..."),
-                                       'printer')
+            n = Notify.Notification.new (_("Configuring new printer"),
+                                         _("Please wait..."),
+                                        'printer')
             n.set_timeout (TIMEOUT + 5000)
-            n.set_data ('closed', False)
+            n.closed = False
             n.connect ('closed', self.on_notification_closed)
             n.show ()
             self.notification = n
 
         self.getting_ready += 1
-        gobject.timeout_add_seconds (TIMEOUT, self.timeout_ready)
+        GLib.timeout_add_seconds (TIMEOUT, self.timeout_ready)
 
     def on_notification_closed (self, notification):
-        notification.set_data ('closed', True)
+        notification.closed = True
 
     def timeout_ready (self):
         if self.getting_ready > 0:
             self.getting_ready -= 1
         if (self.getting_ready == 0 and
             self.notification and
-            not self.notification.get_data ('closed')):
+            not getattr (self.notification, 'closed', None)):
             self.notification.close ()
 
         return False
@@ -124,10 +121,10 @@ class NewPrinterNotification(dbus.service.Object):
                 text = _("No printer driver for %s.") % device
             else:
                 text = _("No driver for this printer.")
-            n = pynotify.Notification (title, text, 'printer')
-            if "actions" in pynotify.get_server_caps():
-                n.set_urgency (pynotify.URGENCY_CRITICAL)
-                n.set_timeout (pynotify.EXPIRES_NEVER)
+            n = Notify.Notification.new (title, text, 'printer')
+            if "actions" in Notify.get_server_caps():
+                n.set_urgency (Notify.Urgency.CRITICAL)
+                n.set_timeout (Notify.EXPIRES_NEVER)
                 n.add_action ("setup-printer", _("Search"),
                               lambda x, y:
                                   self.setup_printer (x, y, name, devid))
@@ -172,13 +169,14 @@ class NewPrinterNotification(dbus.service.Object):
             if len (missing_pkgs) > 0:
                 pkgs = reduce (lambda x,y: x + ", " + y, missing_pkgs)
                 title = _("Install printer driver")
-                text = _("`%s' requires driver installation: %s.") % (name, pkgs)
-                n = pynotify.Notification (title, text)
+                text = (_("`%s' requires driver installation: %s.") %
+                        (name, pkgs))
+                n = Notify.Notification.new (title, text, 'printer')
                 import installpackage
-                if "actions" in pynotify.get_server_caps():
+                if "actions" in Notify.get_server_caps():
                     try:
                         self.packagekit = installpackage.PackageKit ()
-                        n.set_timeout (pynotify.EXPIRES_NEVER)
+                        n.set_timeout (Notify.EXPIRES_NEVER)
                         n.add_action ("install-driver", _("Install"),
                                       lambda x, y:
                                           self.install_driver (x, y,
@@ -196,9 +194,9 @@ class NewPrinterNotification(dbus.service.Object):
             elif status == self.STATUS_SUCCESS:
                 devid = "MFG:%s;MDL:%s;DES:%s;CMD:%s;" % (mfg, mdl, des, cmd)
                 text = _("`%s' is ready for printing.") % name
-                n = pynotify.Notification (title, text)
-                if "actions" in pynotify.get_server_caps():
-                    n.set_urgency (pynotify.URGENCY_NORMAL)
+                n = Notify.Notification.new (title, text, 'printer')
+                if "actions" in Notify.get_server_caps():
+                    n.set_urgency (Notify.Urgency.NORMAL)
                     n.add_action ("test-page", _("Print test page"),
                                   lambda x, y:
                                       self.print_test_page (x, y, name))
@@ -208,16 +206,16 @@ class NewPrinterNotification(dbus.service.Object):
                 devid = "MFG:%s;MDL:%s;DES:%s;CMD:%s;" % (mfg, mdl, des, cmd)
                 text = (_("`%s' has been added, using the `%s' driver.") %
                         (name, driver))
-                n = pynotify.Notification (title, text, 'printer')
-                if "actions" in pynotify.get_server_caps():
-                    n.set_urgency (pynotify.URGENCY_CRITICAL)
+                n = Notify.Notification.new (title, text, 'printer')
+                if "actions" in Notify.get_server_caps():
+                    n.set_urgency (Notify.Urgency.CRITICAL)
                     n.add_action ("test-page", _("Print test page"),
                                   lambda x, y:
                                       self.print_test_page (x, y, name, devid))
                     n.add_action ("find-driver", _("Find driver"),
                                   lambda x, y: 
                                   self.find_driver (x, y, name, devid))
-                    n.set_timeout (pynotify.EXPIRES_NEVER)
+                    n.set_timeout (Notify.EXPIRES_NEVER)
                 else:
                     self.configure (None, None, name)
 
@@ -273,11 +271,11 @@ class NewPrinterNotification(dbus.service.Object):
 
     def collect_exit_code (self, pid):
         # We do this with timers instead of signals because we already
-        # have gobject imported, but don't (yet) import signal;
+        # have GLib imported, but don't (yet) import signal;
         # let's try not to inflate the process size.
         import os
         try:
-            print "Waiting for child %d" % pid
+            print("Waiting for child %d" % pid)
             (pid, status) = os.waitpid (pid, os.WNOHANG)
             if pid == 0:
                 # Run this timer again.
@@ -289,11 +287,11 @@ class NewPrinterNotification(dbus.service.Object):
 
 PROGRAM_NAME="system-config-printer-applet"
 def show_help ():
-    print "usage: %s [--help|--version|--debug]" % PROGRAM_NAME
+    print("usage: %s [--help|--version|--debug]" % PROGRAM_NAME)
 
 def show_version ():
     import config
-    print "%s %s" % (PROGRAM_NAME, config.VERSION)
+    print("%s %s" % (PROGRAM_NAME, config.VERSION))
     
 ####
 #### Main program entry
@@ -322,15 +320,30 @@ class RunLoop:
         self.session_bus = session_bus
         self.loop = loop
         self.timer = None
-        system_bus.add_signal_receiver (self.handle_dbus_signal,
-                                        path=self.DBUS_PATH,
-                                        dbus_interface=self.DBUS_IFACE)
+        try:
+            system_bus.add_signal_receiver (self.handle_dbus_signal,
+                                            path=self.DBUS_PATH,
+                                            dbus_interface=self.DBUS_IFACE)
+        except dbus.DBusException as e:
+            try:
+                print ("%s: failed to add D-Bus signal receiver: %s" \
+                        % (PROGRAM_NAME, e), file=sys.stderr)
+            finally:
+                sys.exit (1)
+
         self.check_for_jobs ()
 
     def remove_signal_receiver (self):
-        self.system_bus.remove_signal_receiver (self.handle_dbus_signal,
-                                                path=self.DBUS_PATH,
-                                                dbus_interface=self.DBUS_IFACE)
+        try:
+            self.system_bus.remove_signal_receiver (self.handle_dbus_signal,
+                                                    path=self.DBUS_PATH,
+                                                    dbus_interface=self.DBUS_IFACE)
+        except dbus.DBusException as e:
+            try:
+                print ("%s: failed to remove D-Bus signal receiver: %s" \
+                        % (PROGRAM_NAME, e), file=sys.stderr)
+            except:
+                pass
 
     def run (self):
         self.loop.run ()
@@ -338,18 +351,18 @@ class RunLoop:
     def __del__ (self):
         self.remove_signal_receiver ()
         if self.timer:
-            gobject.source_remove (self.timer)
+            GLib.source_remove (self.timer)
 
     def handle_dbus_signal (self, *args):
         if self.timer:
-            gobject.source_remove (self.timer)
-        self.timer = gobject.timeout_add (200, self.check_for_jobs)
+            GLib.source_remove (self.timer)
+        self.timer = GLib.timeout_add (200, self.check_for_jobs)
 
     def check_for_jobs (self, *args):
         debugprint ("checking for jobs")
         if any_jobs ():
-            if self.timer != None:
-                gobject.source_remove (self.timer)
+            if self.timer is not None:
+                GLib.source_remove (self.timer)
 
             self.remove_signal_receiver ()
 
@@ -360,9 +373,9 @@ class RunLoop:
                 iface = dbus.Interface (obj, PRINTING_IFACE)
                 path = iface.JobApplet ()
                 debugprint ("Job applet is %s" % path)
-            except dbus.DBusException, e:
+            except dbus.DBusException as e:
                 try:
-                    print e
+                    print(e)
                 except:
                     pass
 
@@ -391,10 +404,10 @@ if __name__ == '__main__':
             set_debugging (True)
 
     # Must be done before connecting to D-Bus (for some reason).
-    if not pynotify.init (PROGRAM_NAME):
+    if not Notify.init (PROGRAM_NAME):
         try:
-            print >> sys.stderr, ("%s: unable to initialize pynotify" %
-                                  PROGRAM_NAME)
+            print(("%s: unable to initialize pynotify" %
+                                  PROGRAM_NAME), file=sys.stderr)
         except:
             pass
 
@@ -403,8 +416,8 @@ if __name__ == '__main__':
         system_bus = dbus.SystemBus()
     except:
         try:
-            print >> sys.stderr, ("%s: failed to connect to system D-Bus" %
-                                  PROGRAM_NAME)
+            print(("%s: failed to connect to system D-Bus" %
+                                  PROGRAM_NAME), file=sys.stderr)
         finally:
             sys.exit (1)
 
@@ -414,8 +427,8 @@ if __name__ == '__main__':
         session_bus.add_signal_receiver (monitor_session)
     except:
         try:
-            print >> sys.stderr, ("%s: failed to connect to "
-                                  "session D-Bus" % PROGRAM_NAME)
+            print(("%s: failed to connect to "
+                                  "session D-Bus" % PROGRAM_NAME), file=sys.stderr)
         finally:
             sys.exit (1)
 
@@ -423,24 +436,24 @@ if __name__ == '__main__':
         NewPrinterNotification(system_bus, session_bus)
     except:
         try:
-            print >> sys.stderr, ("%s: failed to start "
+            print(("%s: failed to start "
                                   "NewPrinterNotification service" %
-                                  PROGRAM_NAME)
+                                  PROGRAM_NAME), file=sys.stderr)
         except:
             pass
 
     try:
         cupshelpers.installdriver.set_debugprint_fn (debugprint)
         cupshelpers.installdriver.PrinterDriversInstaller(system_bus)
-    except Exception, e:
+    except Exception as e:
         try:
-            print >> sys.stderr, ("%s: failed to start "
+            print(("%s: failed to start "
                                   "PrinterDriversInstaller service: "
-                                  "%s" % (PROGRAM_NAME, e))
+                                  "%s" % (PROGRAM_NAME, e)), file=sys.stderr)
         except:
             pass
 
-    loop = gobject.MainLoop ()
+    loop = GObject.MainLoop ()
     runloop = RunLoop (session_bus, system_bus, loop)
     try:
         runloop.run ()
