@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2012 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Authors: Tim Waugh <twaugh@redhat.com>, Vincent Untz
 
@@ -19,12 +19,7 @@
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
-import gobject
 import os
-
-import asyncipp
-import asyncpk1
-import config
 from debug import *
 import debug
 
@@ -84,18 +79,25 @@ class _AsyncMethodCall:
         kwds['reply_handler'] = self.reply_handler
         kwds['error_handler'] = self.error_handler
         kwds['auth_handler'] = self.auth_handler
+        debugprint ("%s: calling %s" % (self, self._fn))
         self._fn (*args, **kwds)
 
     def reply_handler (self, *args):
         if not self._destroyed:
+            debugprint ("%s: to reply_handler at %s" % (self,
+                                                        self._reply_handler))
             self._reply_handler (self, self._reply_data, *args)
 
     def error_handler (self, *args):
         if not self._destroyed:
+            debugprint ("%s: to error_handler at %s" % (self,
+                                                        self._error_handler))
             self._error_handler (self, self._error_data, *args)
 
     def auth_handler (self, *args):
         if not self._destroyed:
+            debugprint ("%s: to auth_handler at %s" % (self,
+                                                       self._auth_handler))
             self._auth_handler (self, self.auth_data, *args)
 
 ######
@@ -112,7 +114,7 @@ class Connection(SemanticOperations):
         self._destroyed = False
 
         # Decide whether to use direct IPP or PolicyKit.
-        if host == None:
+        if host is None:
             host = cups.getServer()
         use_pk = ((host.startswith ('/') or host == 'localhost') and
                   os.getuid () != 0)
@@ -128,6 +130,7 @@ class Connection(SemanticOperations):
 
         if use_pk and try_as_root:
             debugprint ("Using polkit-1 connection class")
+            import asyncpk1
             c = asyncpk1.PK1Connection (reply_handler=subst_reply_handler,
                                         error_handler=subst_error_handler,
                                         host=host, port=port,
@@ -136,6 +139,7 @@ class Connection(SemanticOperations):
             self._conn = c
         else:
             debugprint ("Using IPP connection class")
+            import asyncipp
             c = asyncipp.IPPAuthConnection (reply_handler=subst_reply_handler,
                                             error_handler=subst_error_handler,
                                             auth_handler=subst_auth_handler,
@@ -199,6 +203,7 @@ class Connection(SemanticOperations):
             del self._methodcalls[i]
             args = args[1:]
         if reply_handler and not self._destroyed:
+            debugprint ("%s: chaining up to %s" % (self, reply_handler))
             reply_handler (self, *args)
 
     def _subst_error_handler (self, methodcall, error_handler, *args):
@@ -208,6 +213,7 @@ class Connection(SemanticOperations):
             del self._methodcalls[i]
             args = args[1:]
         if error_handler and not self._destroyed:
+            debugprint ("%s: chaining up to %s" % (self, error_handler))
             error_handler (self, *args)
 
     def _subst_auth_handler (self, methodcall, auth_handler, prompt, method, resource):
@@ -216,6 +222,7 @@ class Connection(SemanticOperations):
             i = self._methodcalls.index (methodcall)
             del self._methodcalls[i]
         if auth_handler and not self._destroyed:
+            debugprint ("%s: chaining up to %s" % (self, auth_handler))
             auth_handler (prompt, self, method, resource)
 
     def set_auth_info (self, password):
@@ -224,8 +231,8 @@ class Connection(SemanticOperations):
 
 if __name__ == "__main__":
     # Demo
+    from gi.repository import GObject
     set_debugging (True)
-    gobject.threads_init ()
 
     class Test:
         def __init__ (self, quit):
@@ -247,15 +254,15 @@ if __name__ == "__main__":
                                    error_handler=self.getDevices_error)
 
         def getDevices_reply (self, conn, result):
-            print conn, result
+            print (conn, result)
             self.destroy ()
 
         def getDevices_error (self, conn, exc):
-            print repr (exc)
+            print (repr (exc))
             self.destroy ()
 
     t = Test (False)
-    loop = gobject.MainLoop ()
+    loop = GObject.MainLoop ()
     t.getDevices ()
     t.destroy ()
 
