@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
-## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2012, 2013, 2014 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Authors: Tim Waugh <twaugh@redhat.com>, Vincent Untz
 
@@ -20,8 +20,12 @@
 
 import cups
 import dbus
-import gobject
-import gtk
+from functools import reduce
+try:
+    from gi.repository import Gdk
+    from gi.repository import Gtk
+except:
+    pass
 import os
 import sys
 import tempfile
@@ -75,12 +79,13 @@ class _PK1AsyncMethodCall:
         pk_method = proxy.get_dbus_method (self._pk_method_name)
 
         try:
+            debugprint ("%s: calling %s" % (self, pk_method))
             pk_method (*self._pk_args,
                         reply_handler=self._pk_reply_handler,
                         error_handler=self._pk_error_handler,
                         timeout=3600)
-        except TypeError, e:
-            debugprint ("Type error in PK call: %s" % e)
+        except TypeError as e:
+            debugprint ("Type error in PK call: %s" % repr (e))
             self.call_fallback_fn ()
 
     def _destroy (self):
@@ -102,13 +107,21 @@ class _PK1AsyncMethodCall:
             return
 
         if str (error) == '':
-            gtk.gdk.threads_enter ()
+            try:
+                Gdk.threads_enter ()
+            except:
+                pass
+            debugprint ("%s: no error, calling reply handler %s" %
+                        (self, self._client_reply_handler))
             self._client_reply_handler (self._conn, self._unpack_fn (*args))
-            gtk.gdk.threads_leave ()
+            try:
+                Gdk.threads_leave ()
+            except:
+                pass
             self._destroy ()
             return
 
-        debugprint ("PolicyKit method failed with: %s" % repr (str (error)))
+        debugprint ("PolicyKit method failed with: %s" % repr (error))
         self.call_fallback_fn ()
 
     def _pk_error_handler (self, exc):
@@ -117,26 +130,37 @@ class _PK1AsyncMethodCall:
 
         if exc.get_dbus_name () == CUPS_PK_NEED_AUTH:
             exc = cups.IPPError (cups.IPP_NOT_AUTHORIZED, 'pkcancel')
-            gtk.gdk.threads_enter ()
+            try:
+                Gdk.threads_enter ()
+            except:
+                pass
+            debugprint ("%s: no auth, calling error handler %s" %
+                        (self, self._client_error_handler))
             self._client_error_handler (self._conn, exc)
-            gtk.gdk.threads_leave ()
+            try:
+                Gdk.threads_leave ()
+            except:
+                pass
             self._destroy ()
             return
 
         debugprint ("PolicyKit call to %s did not work: %s" %
-                    (self._pk_method_name, exc))
+                    (self._pk_method_name, repr (exc)))
         self.call_fallback_fn ()
 
     def call_fallback_fn (self):
         # Make the 'connection' parameter consistent with PK callbacks.
         self._fallback_kwds["reply_handler"] = self._ipp_reply_handler
         self._fallback_kwds["error_handler"] = self._ipp_error_handler
+        debugprint ("%s: calling %s" % (self, self._fallback_fn))
         self._fallback_fn (*self._fallback_args, **self._fallback_kwds)
 
     def _ipp_reply_handler (self, conn, *args):
         if self._destroyed:
             return
 
+        debugprint ("%s: chaining up to %s" % (self,
+                                               self._client_reply_handler))
         self._client_reply_handler (self._conn, *args)
         self._destroy ()
 
@@ -144,6 +168,8 @@ class _PK1AsyncMethodCall:
         if self._destroyed:
             return
 
+        debugprint ("%s: chaining up to %s" % (self,
+                                               self._client_error_handler))
         self._client_error_handler (self._conn, *args)
         self._destroy ()
 
@@ -175,20 +201,20 @@ class _WriteToTmpFile:
 
     def reply_handler (self, conn, none):
         tmpfd = os.open (self._filename, os.O_RDONLY)
-        tmpfile = os.fdopen (tmpfd, 'r')
-        if self._kwds.has_key ("fd"):
+        tmpfile = os.fdopen (tmpfd, 'rt')
+        if "fd" in self._kwds:
             fd = self._kwds["fd"]
             os.lseek (fd, 0, os.SEEK_SET)
             line = tmpfile.readline ()
             while line != '':
-                os.write (fd, line)
-                line = tempfile.readline ()
+                os.write (fd, line.encode('UTF-8'))
+                line = tmpfile.readline ()
         else:
             file_object = self._kwds["file"]
             file_object.seek (0)
             line = tmpfile.readline ()
             while line != '':
-                file_object.write (line)
+                file_object.write (line.encode('UTF-8'))
                 line = tmpfile.readline ()
 
         tmpfile.close ()
@@ -216,7 +242,7 @@ class PK1Connection:
             self._system_bus = None
 
         global _DevicesGet_uses_new_api
-        if _DevicesGet_uses_new_api == None and self._system_bus:
+        if _DevicesGet_uses_new_api is None and self._system_bus:
             try:
                 obj = self._system_bus.get_object(CUPS_PK_NAME, CUPS_PK_PATH)
                 proxy = dbus.Interface (obj, dbus.INTROSPECTABLE_IFACE)
@@ -244,8 +270,8 @@ class PK1Connection:
 
                     break
 
-            except Exception, e:
-                debugprint ("Exception assessing DevicesGet API: %s" % e)
+            except Exception as e:
+                debugprint ("Exception assessing DevicesGet API: %s" % repr (e))
 
         methodtype = type (self._conn.getPrinters)
         bindings = []
@@ -290,15 +316,15 @@ class PK1Connection:
         leftover_kwds = kwds.copy ()
         reply_handler = leftover_kwds.get ("reply_handler")
         error_handler = leftover_kwds.get ("error_handler")
-        if leftover_kwds.has_key ("reply_handler"):
+        if "reply_handler" in leftover_kwds:
             del leftover_kwds["reply_handler"]
-        if leftover_kwds.has_key ("error_handler"):
+        if "error_handler" in leftover_kwds:
             del leftover_kwds["error_handler"]
-        if leftover_kwds.has_key ("auth_handler"):
+        if "auth_handler" in leftover_kwds:
             del leftover_kwds["auth_handler"]
 
         result = [True, reply_handler, error_handler, ()]
-        if self._system_bus == None:
+        if self._system_bus is None:
             return result
 
         tup = []
@@ -316,7 +342,7 @@ class PK1Connection:
                 # ignored.  Skip to the next one.
                 argindex += 1
                 continue
-            except TypeError, e:
+            except TypeError as e:
                 debugprint ("Error converting %s to %s" %
                             (repr (arg), types[argindex]))
                 return result
@@ -325,10 +351,10 @@ class PK1Connection:
             argindex += 1
 
         for kw, default in params[argindex:]:
-            if leftover_kwds.has_key (kw):
+            if kw in leftover_kwds:
                 try:
                     val = self._coerce (types[argindex], leftover_kwds[kw])
-                except TypeError, e:
+                except TypeError as e:
                     debugprint ("Error converting %s to %s" %
                                 (repr (leftover_kwds[kw]), types[argindex]))
                     return result
@@ -341,7 +367,7 @@ class PK1Connection:
             argindex += 1
 
         if leftover_kwds:
-            debugprint ("Leftover keywords: %s" % repr (leftover_kwds.keys ()))
+            debugprint ("Leftover keywords: %s" % repr (list(leftover_kwds.keys ())))
             return result
 
         result[0] = False
@@ -363,7 +389,7 @@ class PK1Connection:
             try:
                 debugprint ("Calling PK method %s" % pk_method_name)
                 asyncmethodcall.call ()
-            except dbus.DBusException, e:
+            except dbus.DBusException as e:
                 debugprint ("D-Bus call failed: %s" % repr (e))
                 use_pycups = True
 
@@ -412,7 +438,7 @@ class PK1Connection:
 
     def _unpack_getDevices_reply (self, dbusdict):
         result_str = dict()
-        for key, value in dbusdict.iteritems ():
+        for key, value in dbusdict.items ():
             if type (key) == dbus.String:
                 result_str[str (key)] = str (value)
             else:
@@ -435,7 +461,7 @@ class PK1Connection:
                 else:
                     device_uri = result_str[keywithaffix]
 
-            if device_uri != None:
+            if device_uri is not None:
                 devices[device_uri] = device_dict
 
             n += 1
@@ -488,7 +514,7 @@ class PK1Connection:
 
         # getFile(resource, filename=None, fd=-1, file=None) -> None
         if use_pycups:
-            if ((len (args) == 0 and kwds.has_key ('resource')) or
+            if ((len (args) == 0 and 'resource' in kwds) or
                 (len (args) == 1)):
                 can_use_tempfile = True
                 for each in kwds.keys ():
@@ -550,35 +576,34 @@ class PK1Connection:
     ## ...
 
 if __name__ == '__main__':
-    import gtk
-    gobject.threads_init ()
+    from gi.repository import GObject
     from debug import set_debugging
     set_debugging (True)
     class UI:
         def __init__ (self):
-            w = gtk.Window ()
-            v = gtk.VBox ()
+            w = Gtk.Window ()
+            v = Gtk.VBox ()
             w.add (v)
-            b = gtk.Button ("Go")
-            v.pack_start (b)
+            b = Gtk.Button.new_with_label ("Go")
+            v.pack_start (b, False, False, 0)
             b.connect ("clicked", self.button_clicked)
-            b = gtk.Button ("Fetch")
-            v.pack_start (b)
+            b = Gtk.Button.new_with_label ("Fetch")
+            v.pack_start (b, False, False, 0)
             b.connect ("clicked", self.fetch_clicked)
             b.set_sensitive (False)
             self.fetch_button = b
-            b = gtk.Button ("Cancel job")
-            v.pack_start (b)
+            b = Gtk.Button.new_with_label ("Cancel job")
+            v.pack_start (b, False, False, 0)
             b.connect ("clicked", self.cancel_clicked)
             b.set_sensitive (False)
             self.cancel_button = b
-            b = gtk.Button ("Get file")
-            v.pack_start (b)
+            b = Gtk.Button.new_with_label ("Get file")
+            v.pack_start (b, False, False, 0)
             b.connect ("clicked", self.get_file_clicked)
             b.set_sensitive (False)
             self.get_file_button = b
-            b = gtk.Button ("Something harmless")
-            v.pack_start (b)
+            b = Gtk.Button.new_with_label ("Something harmless")
+            v.pack_start (b, False, False, 0)
             b.connect ("clicked", self.harmless_clicked)
             b.set_sensitive (False)
             self.harmless_button = b
@@ -598,7 +623,7 @@ if __name__ == '__main__':
             except AttributeError:
                 pass
 
-            gtk.main_quit ()
+            Gtk.main_quit ()
 
         def button_clicked (self, button):
             if self.conn:
@@ -608,14 +633,14 @@ if __name__ == '__main__':
                                        error_handler=self.connection_error)
 
         def connected (self, conn, result):
-            print "Connected"
+            print("Connected")
             self.fetch_button.set_sensitive (True)
             self.cancel_button.set_sensitive (True)
             self.get_file_button.set_sensitive (True)
             self.harmless_button.set_sensitive (True)
 
         def connection_error (self, conn, error):
-            print "Failed to connect"
+            print("Failed to connect")
             raise error
 
         def fetch_clicked (self, button):
@@ -625,57 +650,57 @@ if __name__ == '__main__':
 
         def got_devices (self, conn, devices):
             if conn != self.conn:
-                print "Ignoring stale reply"
+                print("Ignoring stale reply")
                 return
 
-            print "got devices: %s" % devices
+            print("got devices: %s" % devices)
 
         def get_devices_error (self, conn, exc):
             if conn != self.conn:
-                print "Ignoring stale error"
+                print("Ignoring stale error")
                 return
 
-            print "devices error: %s" % repr (exc)
+            print("devices error: %s" % repr (exc))
 
         def cancel_clicked (self, button):
-            print "Cancel job..."
+            print("Cancel job...")
             self.conn.cancelJob (1,
                                  reply_handler=self.job_canceled,
                                  error_handler=self.cancel_job_error)
 
         def job_canceled (self, conn, none):
             if conn != self.conn:
-                print "Ignoring stale reply for %s" % conn
+                print("Ignoring stale reply for %s" % conn)
                 return
 
-            print "Job canceled"
+            print("Job canceled")
 
         def cancel_job_error (self, conn, exc):
             if conn != self.conn:
-                print "Ignoring stale error for %s" % conn
+                print("Ignoring stale error for %s" % conn)
                 return
 
-            print "cancel error: %s" % repr (exc)
+            print("cancel error: %s" % repr (exc))
 
         def get_file_clicked (self, button):
-            self.my_file = file ("cupsd.conf", "w")
+            self.my_file = open ("cupsd.conf", "w")
             self.conn.getFile ("/admin/conf/cupsd.conf", file=self.my_file,
                                reply_handler=self.got_file,
                                error_handler=self.get_file_error)
 
         def got_file (self, conn, none):
             if conn != self.conn:
-                print "Ignoring stale reply for %s" % conn
+                print("Ignoring stale reply for %s" % conn)
                 return
 
-            print "Got file"
+            print("Got file")
 
         def get_file_error (self, conn, exc):
             if conn != self.conn:
-                print "Ignoring stale error"
+                print("Ignoring stale error")
                 return
 
-            print "get file error: %s" % repr (exc)
+            print("get file error: %s" % repr (exc))
 
         def harmless_clicked (self, button):
             self.conn.getJobs (reply_handler=self.got_jobs,
@@ -683,14 +708,14 @@ if __name__ == '__main__':
 
         def got_jobs (self, conn, result):
             if conn != self.conn:
-                print "Ignoring stale reply from %s" % repr (conn)
+                print("Ignoring stale reply from %s" % repr (conn))
                 return
-            print result
+            print(result)
 
         def get_jobs_error (self, exc):
-            print "get jobs error: %s" % repr (exc)
+            print("get jobs error: %s" % repr (exc))
 
     UI ()
     from dbus.mainloop.glib import DBusGMainLoop
     DBusGMainLoop (set_as_default=True)
-    gtk.main ()
+    Gtk.main ()
