@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
 ## check-device-ids
 
-## Copyright (C) 2010, 2011, 2012 Red Hat, Inc.
+## Copyright (C) 2010, 2011, 2012, 2013 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -25,22 +25,22 @@ import cups
 import cupshelpers
 from cupshelpers.ppds import PPDs, ppdMakeModelSplit
 import sys
+from functools import reduce
 
-cups.setUser ('root')
 c = cups.Connection ()
 
 devices = None
 if len (sys.argv) > 1 and sys.argv[1] == '--help':
-    print "Syntax: check-device-ids <device-make-and-model> <device-id>"
-    print "    or: check-device-ids <device-uri>"
-    print "    or: check-device-ids <queue-name>"
-    print "    or: check-device-ids"
+    print("Syntax: check-device-ids <device-make-and-model> <device-id>")
+    print("    or: check-device-ids <device-uri>")
+    print("    or: check-device-ids <queue-name>")
+    print("    or: check-device-ids")
     sys.exit (1)
 
 SPECIFIC_URI = None
 if len (sys.argv) == 3:
     id_dict = cupshelpers.parseDeviceID (sys.argv[2])
-    if id_dict.has_key ("MFG") and id_dict.has_key ("MDL"):
+    if id_dict.get ("MFG") and id_dict.get ("MDL"):
         devices = { 'user-specified:':
                         { 'device-make-and-model': sys.argv[1],
                           'device-id': sys.argv[2] }
@@ -52,20 +52,24 @@ elif len (sys.argv) == 2:
         # This is a queue name.  Work out the URI from that.
         try:
             attrs = c.getPrinterAttributes (sys.argv[1])
-        except cups.IPPError, (e, m):
-            print "Error getting printer attibutes: %s" % m
+        except cups.IPPError as e:
+            (e, m) = e.args
+            print("Error getting printer attibutes: %s" % m)
             sys.exit (1)
 
         SPECIFIC_URI = attrs['device-uri']
-        print "URI for queue %s is %s" % (sys.argv[1], SPECIFIC_URI)
+        print("URI for queue %s is %s" % (sys.argv[1], SPECIFIC_URI))
 else:
     print ("\nIf you have not already done so, you may get more results\n"
            "by temporarily disabling your firewall (or by allowing\n"
            "incoming UDP packets on port 161).\n")
 
-if devices == None:
+if devices is None:
     if not SPECIFIC_URI:
-        print "Examining connected devices"
+        print("Examining connected devices")
+
+    cups.setUser ('root')
+    c = cups.Connection ()
 
     try:
         if SPECIFIC_URI:
@@ -73,27 +77,27 @@ if devices == None:
             devices = c.getDevices (include_schemes=[scheme])
         else:
             devices = c.getDevices (exclude_schemes=["dnssd", "hal", "hpfax"])
-    except cups.IPPError, (e, m):
+    except cups.IPPError as e:
+        (e, m) = e.args
         if e == cups.IPP_FORBIDDEN:
-            print "Run this as root to examine IDs from attached devices."
+            print("Run this as root to examine IDs from attached devices.")
             sys.exit (1)
-
-        if e == cups.IPP_NOT_AUTHORIZED:
-            print "Not authorized."
+        if e in (cups.IPP_NOT_AUTHORIZED, cups.IPP_AUTHENTICATION_CANCELED):
+            print("Not authorized.")
             sys.exit (1)
 
 if SPECIFIC_URI:
-    if devices.get (SPECIFIC_URI) == None:
+    if devices.get (SPECIFIC_URI) is None:
         devices = { SPECIFIC_URI:
                         { 'device-make-and-model': '',
                           'device-id': ''} }
 if len (devices) == 0:
-    print "No attached devices."
+    print("No attached devices.")
     sys.exit (0)
 
 n = 0
 device_ids = []
-for device, attrs in devices.iteritems ():
+for device, attrs in devices.items ():
     if device.find (":") == -1:
         continue
 
@@ -119,7 +123,7 @@ for device, attrs in devices.iteritems ():
                 devs = []
 
                 def got_device (dev):
-                    if dev != None:
+                    if dev is not None:
                         devs.append (dev)
 
                 import probe_printer
@@ -127,7 +131,7 @@ for device, attrs in devices.iteritems ():
                 pf.hostname = hostname
                 pf.callback_fn = got_device
                 pf._cached_attributes = dict()
-                print "Sending SNMP request to %s for device-id" % hostname
+                print("Sending SNMP request to %s for device-id" % hostname)
                 pf._probe_snmp ()
 
                 for dev in devs:
@@ -140,11 +144,11 @@ for device, attrs in devices.iteritems ():
                         attrs.update ({'device-make-and-model':
                                            dev.make_and_model})
 
-        except Exception, e:
-            print "Exception: %s" % repr (e)
+        except Exception as e:
+            print("Exception: %s" % repr (e))
 
     if not (make_and_model and device_id):
-        print "Skipping %s, insufficient data" % device
+        print("Skipping %s, insufficient data" % device)
         continue
 
     id_fields = cupshelpers.parseDeviceID (device_id)
@@ -153,26 +157,26 @@ for device, attrs in devices.iteritems ():
     n += 1
 
 if not device_ids:
-    print "No Device IDs available."
+    print("No Device IDs available.")
     sys.exit (0)
 
 try:
     bus = dbus.SessionBus ()
 
-    print "Installing relevant drivers using session service"
+    print("Installing relevant drivers using session service")
     try:
         obj = bus.get_object ("org.freedesktop.PackageKit",
                               "/org/freedesktop/PackageKit")
         proxy = dbus.Interface (obj, "org.freedesktop.PackageKit.Modify")
         proxy.InstallPrinterDrivers (0, device_ids,
                                      "hide-finished", timeout=3600)
-    except dbus.exceptions.DBusException, e:
-        print "Ignoring exception: %s" % e
+    except dbus.exceptions.DBusException as e:
+        print("Ignoring exception: %s" % e)
 except dbus.exceptions.DBusException:
     try:
         bus = dbus.SystemBus ()
 
-        print "Installing relevant drivers using system service"
+        print("Installing relevant drivers using system service")
         try:
             obj = bus.get_object ("com.redhat.PrinterDriversInstaller",
                                   "/com/redhat/PrinterDriversInstaller")
@@ -182,13 +186,13 @@ except dbus.exceptions.DBusException:
                 id_dict = cupshelpers.parseDeviceID (device_id)
                 proxy.InstallDrivers (id_dict['MFG'], id_dict['MDL'], '',
                                       timeout=3600)
-        except dbus.exceptions.DBusException, e:
-            print "Ignoring exception: %s" % e
+        except dbus.exceptions.DBusException as e:
+            print("Ignoring exception: %s" % e)
     except dbus.exceptions.DBusException:
-        print "D-Bus not available so skipping package installation"
+        print("D-Bus not available so skipping package installation")
 
 
-print "Fetching driver list"
+print("Fetching driver list")
 ppds = PPDs (c.getPPDs ())
 ppds._init_ids ()
 makes = ppds.getMakes ()
@@ -201,7 +205,7 @@ def driver_uri_to_filename (uri):
         elif uri.startswith ("lsb/opt/"):
             return "/opt/share/ppd/" + uri[8:]
         elif uri.startswith ("lsb/local/"):
-            return "/usr/local/local/share/ppd/" + uri[10:]
+            return "/usr/local/share/ppd/" + uri[10:]
 
         return "/usr/local/share/cups/model/" + uri
 
@@ -231,13 +235,13 @@ def driver_uri_to_pkg (uri):
 
 i = 1
 if sys.stdout.encoding == 'UTF-8':
-    item = unichr (0x251c) + unichr (0x2500) + unichr (0x2500)
-    last = unichr (0x2514) + unichr (0x2500) + unichr (0x2500)
+    item = chr (0x251c) + chr (0x2500) + chr (0x2500)
+    last = chr (0x2514) + chr (0x2500) + chr (0x2500)
 else:
     item = "|--"
     last = "`--"
 
-for device, attrs in devices.iteritems ():
+for device, attrs in devices.items ():
     make_and_model = attrs.get ('device-make-and-model')
     device_id = attrs.get ('device-id')
     if device.find (":") == -1:
@@ -259,11 +263,11 @@ for device, attrs in devices.iteritems ():
         cmd = ""
 
     scheme = device.split (":", 1)[0]
-    print "%s %s (%s): MFG:%s;MDL:%s;%s" % (line, make_and_model,
+    print("%s %s (%s): MFG:%s;MDL:%s;%s" % (line, make_and_model,
                                             scheme,
                                             id_fields['MFG'],
                                             id_fields['MDL'],
-                                            cmd)
+                                            cmd))
     
     try:
         drivers = ppds.ids[id_fields['MFG'].lower ()][id_fields['MDL'].lower ()]
@@ -271,7 +275,7 @@ for device, attrs in devices.iteritems ():
         drivers = []
 
     if i < n:
-        more = unichr (0x2502)
+        more = chr (0x2502)
     else:
         more = " "
 
@@ -281,15 +285,15 @@ for device, attrs in devices.iteritems ():
         j = 1
         for driver in drivers:
             if j < n_drivers:
-                print "%s   %s %s [%s]" % (more, item, driver,
-                                           driver_uri_to_pkg (driver))
+                print("%s   %s %s [%s]" % (more, item, driver,
+                                           driver_uri_to_pkg (driver)))
             else:
-                print "%s   %s %s [%s]" % (more, last, driver,
-                                           driver_uri_to_pkg (driver))
+                print("%s   %s %s [%s]" % (more, last, driver,
+                                           driver_uri_to_pkg (driver)))
 
             j += 1
     else:
-        print "%s   (No drivers)" % more
+        print("%s   (No drivers)" % more)
 
     (mfr, mdl) = ppdMakeModelSplit (make_and_model)
     matches = set (ppds.getInfoFromModel (mfr, mdl))
@@ -308,22 +312,26 @@ for device, attrs in devices.iteritems ():
     for each in missing:
         try:
             ppd_device_id = ppds.getInfoFromPPDName (each).get ('ppd-device-id')
-        except Exception, e:
-            print e
+        except Exception as e:
+            print(e)
             ppd_device_id = None
 
         if ppd_device_id:
-            print "%s       WRONG    %s [%s]" % (more, each,
-                                                 driver_uri_to_pkg (each))
             ppd_id_fields = cupshelpers.parseDeviceID (ppd_device_id)
+        else:
+            ppd_id_fields = {}
+
+        if ppd_id_fields.get ("MFG") and ppd_id_fields.get ("MDL"):
+            print("%s       WRONG    %s [%s]" % (more, each,
+                                                 driver_uri_to_pkg (each)))
             for field in ["MFG", "MDL"]:
                 value = id_fields[field]
                 ppd_value = ppd_id_fields[field]
                 if value.lower () != ppd_value.lower ():
-                    print "%s                      %s:%s;" % (more, field, ppd_value)
-                    print "%s                should be:%s;" % (more, value)
+                    print("%s                      %s:%s;" % (more, field, ppd_value))
+                    print("%s                should be:%s;" % (more, value))
         else:
-            print "%s       MISSING  %s [%s]" % (more, each,
-                                                 driver_uri_to_pkg (each))
+            print("%s       MISSING  %s [%s]" % (more, each,
+                                                 driver_uri_to_pkg (each)))
 
     i += 1
