@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
-## Copyright (C) 2010, 2011, 2012 Red Hat, Inc.
+## Copyright (C) 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -20,10 +20,12 @@
 
 import asyncconn
 import cups
-import gobject
-import gtk
+from gi.repository import GLib
+from gi.repository import Gdk
+from gi.repository import Gtk
 import os
-import tempfile
+from shutil import copyfileobj
+from tempfile import NamedTemporaryFile
 from debug import *
 
 cups.require ("1.9.50")
@@ -47,7 +49,7 @@ class PPDCache:
             self._cups.destroy ()
 
     def fetch_ppd (self, name, callback, check_uptodate=True):
-        if check_uptodate and self._modtimes.has_key (name):
+        if check_uptodate and name in self._modtimes:
             # We have getPPD3 so we can check whether the PPD is up to
             # date.
             debugprint ("%s: check if %s is up to date" % (self, name))
@@ -61,7 +63,7 @@ class PPDCache:
 
         try:
             f = self._cache[name]
-        except RuntimeError, e:
+        except RuntimeError as e:
             self._schedule_callback (callback, name, None, e)
             return
         except KeyError:
@@ -84,18 +86,14 @@ class PPDCache:
         # PPD object from it, then remove the file.  This way we don't
         # leave temporary files around even though we are caching...
         f.seek (0)
-        (tmpfd, tmpfname) = tempfile.mkstemp ()
-        tmpf = file (tmpfname, "w")
-        tmpf.writelines (f.readlines ())
-        del tmpf
-        os.close (tmpfd)
-        try:
-            ppd = cups.PPD (tmpfname)
-            os.unlink (tmpfname)
-            self._schedule_callback (callback, name, ppd, None)
-        except Exception, e:
-            os.unlink (tmpfname)
-            self._schedule_callback (callback, name, None, e)
+        with NamedTemporaryFile () as tmpf:
+            copyfileobj (f, tmpf)
+
+            try:
+                ppd = cups.PPD (tmpf.file)
+                self._schedule_callback (callback, name, ppd, None)
+            except Exception as e:
+                self._schedule_callback (callback, name, None, e)
 
     def _connect (self, callback=None):
         self._connecting = True
@@ -106,14 +104,20 @@ class PPDCache:
 
     def _got_ppd (self, connection, name, result, callback):
         if isinstance (result, Exception):
-            self._schedule_callback (callback, name, result, None)
+            self._schedule_callback (callback, name, none, result)
         else:
             # Store an open file object, then remove the actual file.
             # This way we don't leave temporary files around.
-            self._cache[name] = file (result)
-            debugprint ("%s: caching %s (fd %d)" % (self, result,
-                                                    self._cache[name].fileno()))
-            os.unlink (result)
+            try:
+                self._cache[name] = open (result, "rb")
+                debugprint ("%s: caching %s (fd %d)" %
+                            (self, result,
+                             self._cache[name].fileno()))
+                os.unlink (result)
+            except IOError as exc:
+                self._schedule_callback (callback, name, None, exc)
+                return
+
             self.fetch_ppd (name, callback)
 
     def _got_ppd3 (self, connection, name, result, callback):
@@ -136,17 +140,19 @@ class PPDCache:
                 # file.  This way we don't leave temporary files
                 # around.
                 try:
-                    self._cache[name] = file (filename)
+                    self._cache[name] = open (filename, "rb")
                     debugprint ("%s: caching %s (fd %d) "
                                 "(%s) - %s" % (self, filename,
                                                self._cache[name].fileno (),
                                                modtime, status))
                     os.unlink (filename)
                     self._modtimes[name] = modtime
-                except IOError:
+                except IOError as exc:
                     # File disappeared?
                     debugprint ("%s: file %s disappeared? Unable to cache it"
                                 % (self, filename))
+                    self._schedule_callback (callback, name, None, exc)
+                    return
 
             # Now fetch it from our own cache.
             self.fetch_ppd (name, callback, check_uptodate=False)
@@ -169,25 +175,25 @@ class PPDCache:
 
     def _schedule_callback (self, callback, name, result, exc):
         def cb_func (callback, name, result, exc):
-            gtk.gdk.threads_enter ()
+            Gdk.threads_enter ()
             callback (name, result, exc)
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
             return False
 
-        gobject.idle_add (cb_func, callback, name, result, exc)
+        GLib.idle_add (cb_func, callback, name, result, exc)
 
 if __name__ == "__main__":
     import sys
     from debug import *
+    from gi.repository import GObject
     set_debugging (True)
-    gobject.threads_init ()
-    gtk.gdk.threads_init ()
-    loop = gobject.MainLoop ()
+    Gdk.threads_init ()
+    loop = GObject.MainLoop ()
 
     def signal (name, result, exc):
         debugprint ("**** %s" % name)
-        debugprint (result)
-        debugprint (exc)
+        debugprint (repr (result))
+        debugprint (repr (exc))
 
     c = cups.Connection ()
     printers = c.getPrinters ()
@@ -199,6 +205,6 @@ if __name__ == "__main__":
         cache.fetch_ppd (p, signal)
 
     if p:
-        gobject.timeout_add_seconds (1, cache.fetch_ppd, p, signal)
-        gobject.timeout_add_seconds (5, loop.quit)
+        GLib.timeout_add_seconds (1, cache.fetch_ppd, p, signal)
+        GLib.timeout_add_seconds (5, loop.quit)
     loop.run ()
