@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
 ## system-config-printer
 
-## Copyright (C) 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -20,8 +20,8 @@
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import dbus
-import gobject
-import gtk
+from gi.repository import GObject
+from gi.repository import Gtk
 import cupshelpers
 
 import cups
@@ -29,9 +29,11 @@ cups.require ("1.9.52")
 
 import asyncconn
 from debug import debugprint
-from gettext import gettext as _
+import config
+import gettext
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 
-class PPDsLoader(gobject.GObject):
+class PPDsLoader(GObject.GObject):
     """
     1. If PackageKit support is available, and this is a local server,
     try to use PackageKit to install relevant drivers.  We do this
@@ -48,13 +50,13 @@ class PPDsLoader(gobject.GObject):
     """
 
     __gsignals__ = {
-        'finished': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
+        'finished': (GObject.SignalFlags.RUN_LAST, None, ())
         }
 
     def __init__ (self, device_id=None, parent=None, device_uri=None,
                   host=None, encryption=None, language=None,
                   device_make_and_model=None):
-        gobject.GObject.__init__ (self)
+        GObject.GObject.__init__ (self)
         debugprint ("+%s" % self)
         self._device_id = device_id
         self._device_uri = device_uri
@@ -71,7 +73,8 @@ class PPDsLoader(gobject.GObject):
 
         self._ppdsmatch_result = None
         self._jockey_queried = False
-        self._local_cups = (self._host == None or
+        self._jockey_has_answered = False
+        self._local_cups = (self._host is None or
                             self._host == "localhost" or
                             self._host[0] == '/')
         try:
@@ -81,12 +84,11 @@ class PPDsLoader(gobject.GObject):
             self._bus = None
 
         fmt = _("Searching")
-        self._dialog = gtk.MessageDialog (parent=parent,
-                                          flags=gtk.DIALOG_MODAL |
-                                          gtk.DIALOG_DESTROY_WITH_PARENT,
-                                          type=gtk.MESSAGE_INFO,
-                                          buttons=gtk.BUTTONS_CANCEL,
-                                          message_format=fmt)
+        self._dialog = Gtk.MessageDialog (parent=parent,
+                                          modal=True, destroy_with_parent=True,
+                                          message_type=Gtk.MessageType.INFO,
+                                          buttons=Gtk.ButtonsType.CANCEL,
+                                          text=fmt)
 
         self._dialog.format_secondary_text (_("Searching for drivers"))
 
@@ -98,7 +100,11 @@ class PPDsLoader(gobject.GObject):
         if self._device_id:
             self._devid_dict = cupshelpers.parseDeviceID (self._device_id)
 
-        if self._local_cups and self._device_id and self._bus:
+        if (self._local_cups and
+            self._device_id and
+            self._devid_dict["MFG"] and
+            self._devid_dict["MDL"] and
+            self._bus):
             self._gpk_device_id = "MFG:%s;MDL:%s;" % (self._devid_dict["MFG"],
                                                       self._devid_dict["MDL"])
             self._query_packagekit ()
@@ -130,6 +136,7 @@ class PPDsLoader(gobject.GObject):
         return self._ppdsmatch_result
 
     def get_error (self):
+        debugprint ("%s: stored error is %s" % (self, repr (self._exc)))
         return self._exc
 
     def _dialog_response (self, dialog, response):
@@ -167,7 +174,7 @@ class PPDsLoader(gobject.GObject):
                                          self._device_make_and_model)
 
             ppdnamelist = ppds.\
-                orderPPDNamesByPreference (fit.keys (),
+                orderPPDNamesByPreference (list(fit.keys ()),
                                            self._installed_files,
                                            devid=self._devid_dict,
                                            fit=fit)
@@ -177,6 +184,8 @@ class PPDsLoader(gobject.GObject):
             if (self._bus and
                 not fit[ppdname].startswith ("exact") and
                 not self._jockey_queried and
+                self._devid_dict["MFG"] and
+                self._devid_dict["MDL"] and
                 self._local_cups):
                 # Try to install packages using jockey if
                 # - there's no appropriate driver (PPD) locally available
@@ -187,7 +196,7 @@ class PPDsLoader(gobject.GObject):
 
         conn.destroy ()
         self._conn = None
-        if self._dialog != None:
+        if self._dialog is not None:
             self._dialog.destroy ()
             self._dialog = None
 
@@ -197,7 +206,8 @@ class PPDsLoader(gobject.GObject):
         conn.destroy ()
         self._conn = None
         self._ppds = None
-        if self._dialog != None:
+        self._exc = exc
+        if self._dialog is not None:
             self._dialog.destroy ()
             self._dialog = None
 
@@ -205,11 +215,6 @@ class PPDsLoader(gobject.GObject):
 
     def _query_packagekit (self):
         debugprint ("Asking PackageKit to install drivers")
-        try:
-            xid = self._parent.window.xid
-        except:
-            xid = 0
-
         try:
             obj = self._bus.get_object ("org.freedesktop.PackageKit",
                                         "/org/freedesktop/PackageKit")
@@ -217,14 +222,14 @@ class PPDsLoader(gobject.GObject):
             resources = [self._gpk_device_id]
             interaction = "hide-finished"
             debugprint ("Calling InstallPrinterDrivers (%s, %s, %s)" %
-                        (repr (xid), repr (resources), repr (interaction)))
-            proxy.InstallPrinterDrivers (dbus.UInt32 (xid),
+                        (repr (0), repr (resources), repr (interaction)))
+            proxy.InstallPrinterDrivers (dbus.UInt32 (0),
                                          resources, interaction,
                                          reply_handler=self._packagekit_reply,
                                          error_handler=self._packagekit_error,
                                          timeout=3600)
-        except Exception, e:
-            debugprint ("Failed to talk to PackageKit: %s" % e)
+        except Exception as e:
+            debugprint ("Failed to talk to PackageKit: %s" % repr (e))
             if self._dialog:
                 self._dialog.show_all ()
                 self._query_cups ()
@@ -237,7 +242,7 @@ class PPDsLoader(gobject.GObject):
             self._query_cups ()
 
     def _packagekit_error (self, exc):
-        debugprint ("Got PackageKit error: %s" % exc)
+        debugprint ("Got PackageKit error: %s" % repr (exc))
         if self._dialog:
             self._dialog.show_all ()
             self._query_cups ()
@@ -251,11 +256,12 @@ class PPDsLoader(gobject.GObject):
                                       reply_handler=self._jockey_reply,
                                       error_handler=self._jockey_error,
                                       timeout=3600)
-        except Exception, e:
+        except Exception as e:
             self._jockey_error (e)
 
     def _jockey_reply (self, conn, result):
         debugprint ("Got Jockey result: %s" % repr (result))
+        self._jockey_has_answered = True
         try:
             self._installed_files = result[1]
         except:
@@ -263,30 +269,28 @@ class PPDsLoader(gobject.GObject):
         self._query_cups ()
 
     def _jockey_error (self, exc):
-        debugprint ("Got Jockey error: %s" % exc)
+        debugprint ("Got Jockey error: %s" % repr (exc))
         if self._need_requery_cups:
             self._query_cups ()
         else:
-            if self._conn != None:
+            if self._conn is not None:
                 self._conn.destroy ()
                 self._conn = None
 
-            if self._dialog != None:
+            if self._dialog is not None:
                 self._dialog.destroy ()
                 self._dialog = None
 
             self.emit ('finished')
 
-gobject.type_register(PPDsLoader)
-
 if __name__ == "__main__":
     class Foo:
         def __init__ (self):
-            w = gtk.Window ()
-            b = gtk.Button ("Go")
+            w = Gtk.Window ()
+            b = Gtk.Button.new_with_label ("Go")
             w.add (b)
             b.connect ('clicked', self.go)
-            w.connect ('delete-event', gtk.main_quit)
+            w.connect ('delete-event', Gtk.main_quit)
             w.show_all ()
             self._window = w
 
@@ -298,18 +302,16 @@ if __name__ == "__main__":
 
         def ppds_loaded (self, ppdsloader):
             self._window.destroy ()
-            gtk.main_quit ()
+            Gtk.main_quit ()
             exc = ppdsloader.get_error ()
-            print exc
+            print(exc)
             ppds = ppdsloader.get_ppds ()
-            if ppds != None:
-                print len (ppds)
+            if ppds is not None:
+                print(len (ppds))
 
             ppdsloader.destroy ()
 
-    import gobject
     from debug import set_debugging
     set_debugging (True)
-    gobject.threads_init ()
     Foo ()
-    gtk.main ()
+    Gtk.main ()
