@@ -1,8 +1,8 @@
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2014 Red Hat, Inc.
 ## Copyright (C) 2006 Florian Festi <ffesti@redhat.com>
-## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2007, 2008, 2009, 2014 Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -22,13 +22,14 @@ import cupshelpers
 from debug import *
 import errno
 import socket, time
-import gtk
+from gi.repository import Gdk
+from gi.repository import Gtk
 from timedops import TimedOperation
 import subprocess
 import threading
-import errno
 import cups
-import gobject
+from gi.repository import GObject
+from gi.repository import GLib
 import smburi
 
 try:
@@ -94,12 +95,12 @@ def open_socket(hostname, port):
         host, port = hostname.split(":", 1)
     except ValueError:
         host = hostname
-        
+
     s = None
     try:
         ai = socket.getaddrinfo(host, port, socket.AF_UNSPEC,
                                 socket.SOCK_STREAM)
-    except socket.gaierror:
+    except (socket.gaierror, socket.error):
         ai = []
 
     for res in ai:
@@ -107,12 +108,12 @@ def open_socket(hostname, port):
         try:
             s = socket.socket(af, socktype, proto)
             s.settimeout(0.5)
-        except socket.error, msg:
+        except socket.error:
             s = None
             continue
         try:
             s.connect(sa)
-        except socket.error, msg:
+        except socket.error:
             s.close()
             s = None
             continue
@@ -129,14 +130,14 @@ class LpdServer:
         s = open_socket(self.hostname, 515)
         if not s:
             return None
-        print name
+        print(name)
         
         try:
-            s.send('\2%s\n' % name) # cmd send job to queue
-            data = s.recv(1024) # receive status
-            print repr(data)
-        except socket.error, msg:
-            print msg
+            s.send(('\2%s\n' % name).encode('UTF-8'))  # cmd send job to queue
+            data = s.recv(1024).decode('UTF-8')  # receive status
+            print(repr(data))
+        except socket.error as msg:
+            print(msg)
             try:
                 s.close ()
             except:
@@ -146,7 +147,7 @@ class LpdServer:
 
         if len(data)>0 and ord(data[0])==0:
             try:
-                s.send('\1\n') # abort job again
+                s.send(b'\1\n')  # abort job again
                 s.close ()
             except:
                 pass
@@ -180,14 +181,14 @@ class LpdServer:
     def probe(self):
         result = []
         for name in self.get_possible_queue_names ():
-            while gtk.events_pending ():
-                gtk.main_iteration ()
+            while Gtk.events_pending ():
+                Gtk.main_iteration ()
 
             if self.stop:
                 break
 
             found = self.probe_queue(name, result)
-            if found == None:
+            if found is None:
                 # Couldn't even connect.
                 break
 
@@ -206,9 +207,9 @@ class BackgroundSmbAuthContext(pysmb.AuthContext):
         pysmb.AuthContext.__init__ (self, *args, **kwargs)
 
     def _do_perform_authentication (self):
-        gtk.gdk.threads_enter ()
+        Gdk.threads_enter ()
         result = pysmb.AuthContext.perform_authentication (self)
-        gtk.gdk.threads_leave ()
+        Gdk.threads_leave ()
         self._do_perform_authentication_result = result
         self._gui_event.set ()
         
@@ -221,7 +222,7 @@ class BackgroundSmbAuthContext(pysmb.AuthContext):
             return pysmb.AuthContext.perform_authentication (self)
 
         self._gui_event.clear ()
-        gobject.timeout_add (1, self._do_perform_authentication)
+        GLib.timeout_add (1, self._do_perform_authentication)
         self._gui_event.wait ()
         return self._do_perform_authentication_result
 
@@ -240,18 +241,18 @@ class PrinterFinder:
 
     def _do_find (self):
         self._cached_attributes = dict()
-        for fn in [self._probe_jetdirect,
+        for fn in [self._probe_hplip,
+                   self._probe_jetdirect,
                    self._probe_ipp,
                    self._probe_snmp,
                    self._probe_lpd,
-                   self._probe_hplip,
                    self._probe_smb]:
             if self.quit:
                 return
 
             try:
                 fn ()
-            except Exception, e:
+            except Exception:
                 nonfatalException ()
 
         # Signal that we've finished.
@@ -270,18 +271,17 @@ class PrinterFinder:
 
     def _probe_snmp (self):
         # Run the CUPS SNMP backend, pointing it at the host.
-        null = file ("/dev/null", "r+")
         try:
             debugprint ("snmp: trying")
             p = subprocess.Popen (args=["/usr/local/libexec/cups/backend/snmp",
                                         self.hostname],
                                   close_fds=True,
-                                  stdin=null,
+                                  stdin=subprocess.DEVNULL,
                                   stdout=subprocess.PIPE,
-                                  stderr=null)
-        except OSError, e:
+                                  stderr=subprocess.DEVNULL)
+        except OSError as e:
             debugprint ("snmp: no good")
-            if e == errno.ENOENT:
+            if e.errno == errno.ENOENT:
                 return
 
             raise
@@ -295,7 +295,7 @@ class PrinterFinder:
             debugprint ("snmp: no good")
             return
 
-        for line in stdout.split ('\n'):
+        for line in stdout.decode ().split ('\n'):
             words = wordsep (line)
             n = len (words)
             if n == 6:
@@ -338,7 +338,7 @@ class PrinterFinder:
                 return
 
             found = lpd.probe_queue (name, [])
-            if found == None:
+            if found is None:
                 # Couldn't even connect.
                 debugprint ("lpd: couldn't connect")
                 break
@@ -355,16 +355,23 @@ class PrinterFinder:
         debugprint ("lpd: done")
 
     def _probe_hplip (self):
-        null = file ("/dev/null", "r+")
+        if ('device-make-and-model' in self._cached_attributes and \
+            self._cached_attributes['device-make-and-model'] != "Unknown" and \
+            not self._cached_attributes['device-make-and-model'].lower ().startswith ("hp") and \
+            not self._cached_attributes['device-make-and-model'].lower ().startswith ("hewlett")):
+            debugprint ("hplip: no good (Non-HP printer: %s)" %
+                        self._cached_attributes['device-make-and-model'])
+            return
+
         try:
             debugprint ("hplip: trying")
             p = subprocess.Popen (args=["hp-makeuri", "-c", self.hostname],
                                   close_fds=True,
-                                  stdin=null,
+                                  stdin=subprocess.DEVNULL,
                                   stdout=subprocess.PIPE,
-                                  stderr=null)
-        except OSError, e:
-            if e == errno.ENOENT:
+                                  stderr=subprocess.DEVNULL)
+        except OSError as e:
+            if e.errno == errno.ENOENT:
                 return
 
             raise
@@ -378,7 +385,7 @@ class PrinterFinder:
             debugprint ("hplip: no good")
             return
 
-        uri = stdout.strip ()
+        uri = stdout.decode ().strip ()
         debugprint ("hplip: uri is %s" % uri)
         if uri.find (":") != -1:
             self._new_device(uri, uri)
@@ -406,9 +413,10 @@ class PrinterFinder:
 
                 try:
                     entries = ctx.opendir (uri).getdents ()
-                except Exception, e:
+                except Exception as e:
                     smbc_auth.failed (e)
-        except RuntimeError, (e, s):
+        except RuntimeError as e:
+            (e, s) = e.args
             if e not in [errno.ENOENT, errno.EACCES, errno.EPERM]:
                 debugprint ("Runtime error: %s" % repr ((e, s)))
         except:
@@ -479,7 +487,7 @@ class PrinterFinder:
             debugprint ("ipp: done")
             return
 
-        for name, queue in printers.iteritems ():
+        for name, queue in printers.items ():
             uri = queue['printer-uri-supported']
             info = queue['printer-info']
             location = queue['printer-location']
@@ -490,14 +498,14 @@ class PrinterFinder:
 if __name__ == '__main__':
     import sys
     if len (sys.argv) < 2:
-        print "Need printer address"
+        print("Need printer address")
         sys.exit (1)
 
     set_debugging (True)
-    loop = gobject.MainLoop ()
+    loop = GObject.MainLoop ()
 
     def display (device):
-        if device == None:
+        if device is None:
             loop.quit ()
 
     addr = sys.argv[1]
