@@ -83,7 +83,7 @@ class AuthContext:
         self.auth_called = False
 
         if self.dialog_shown:
-            d = Gtk.MessageDialog (parent=self.parent,
+            d = Gtk.MessageDialog (transient_for=self.parent,
                                    modal=True, destroy_with_parent=True,
                                    message_type=Gtk.MessageType.ERROR,
                                    buttons=Gtk.ButtonsType.CLOSE)
@@ -119,21 +119,21 @@ class AuthContext:
         label.set_line_wrap (True)
         vbox.pack_start (label, False, False, 0)
 
-        table = Gtk.Table (n_rows=3, n_columns=2)
-        table.set_row_spacings (6)
-        table.set_col_spacings (6)
-        table.attach (Gtk.Label(label=_("Username:")), 0, 1, 0, 1, 0, 0)
+        grid = Gtk.Grid()
+        grid.set_row_spacing (6)
+        grid.set_column_spacing (6)
+        grid.attach (Gtk.Label(label=_("Username:")), 0, 0, 1, 1)
         username_entry = Gtk.Entry ()
-        table.attach (username_entry, 1, 2, 0, 1, 0, 0)
-        table.attach (Gtk.Label(label=_("Domain:")), 0, 1, 1, 2, 0, 0)
+        grid.attach (username_entry, 1, 0, 1, 1)
+        grid.attach (Gtk.Label(label=_("Domain:")), 0, 1, 1, 1)
         domain_entry = Gtk.Entry ()
-        table.attach (domain_entry, 1, 2, 1, 2, 0, 0)
-        table.attach (Gtk.Label(label=_("Password:")), 0, 1, 2, 3, 0, 0)
+        grid.attach (domain_entry, 1, 1, 1, 1)
+        grid.attach (Gtk.Label(label=_("Password:")), 0, 2, 1, 1)
         password_entry = Gtk.Entry ()
         password_entry.set_activates_default (True)
         password_entry.set_visibility (False)
-        table.attach (password_entry, 1, 2, 2, 3, 0, 0)
-        vbox.pack_start (table, False, False, 0)
+        grid.attach (password_entry, 1, 2, 1, 1)
+        vbox.pack_start (grid, False, False, 0)
         hbox.pack_start (vbox, False, False, 0)
         d.vbox.pack_start (hbox, False, False, 0)
         self.dialog_shown = True
