@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## system-config-printer
 
-## Copyright (C) 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -21,7 +21,10 @@
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import dbus.service
-import gobject
+from gi.repository import GObject
+from gi.repository import GLib
+from gi.repository import Gdk
+from gi.repository import Gtk
 import sys
 
 from debug import *
@@ -31,6 +34,7 @@ import cups
 import cupshelpers
 import dnssdresolve
 import jobviewer
+import killtimer
 import newprinter
 import PhysicalDevice
 import ppdcache
@@ -48,63 +52,28 @@ CONFIG_JOBVIEWER_IFACE=CONFIG_IFACE + ".JobViewer"
 g_ppds = None
 g_killtimer = None
 
-class KillTimer:
-    def __init__ (self, timeout=30, killfunc=None):
-        self._timeout = timeout
-        self._killfunc = killfunc
-        self._holds = 0
-        self._add_timeout ()
-
-    def _add_timeout (self):
-        self._timer = gobject.timeout_add_seconds (self._timeout, self._kill)
-
-    def _kill (self):
-        debugprint ("Timeout (%ds), exiting" % self._timeout)
-        if self._killfunc:
-            self._killfunc ()
-        else:
-            sys.exit (0)
-
-    def add_hold (self):
-        if self._holds == 0:
-            debugprint ("Kill timer stopped")
-            gobject.source_remove (self._timer)
-
-        self._holds += 1
-
-    def remove_hold (self):
-        if self._holds > 0:
-            self._holds -= 1
-            if self._holds == 0:
-                debugprint ("Kill timer started")
-                self._add_timeout ()
-
-    def alive (self):
-        if self._holds == 0:
-            gobject.source_remove (self._timer)
-            self._add_timeout ()
-
-class FetchedPPDs(gobject.GObject):
+class FetchedPPDs(GObject.GObject):
     __gsignals__ = {
-        'ready': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
-        'error': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                  [gobject.TYPE_PYOBJECT])
+        'ready': (GObject.SIGNAL_RUN_LAST, None, ()),
+        'error': (GObject.SIGNAL_RUN_LAST, None,
+                  (GObject.TYPE_PYOBJECT,))
         }
 
     def __init__ (self, cupsconn, language):
-        gobject.GObject.__init__ (self)
+        GObject.GObject.__init__ (self)
         self._cupsconn = cupsconn
         self._language = language
         self._ppds = None
 
     def is_ready (self):
-        return self._ppds != None
+        return self._ppds is not None
 
     def get_ppds (self):
         return self._ppds
 
     def run (self):
         debugprint ("FetchPPDs: running")
+        self._ppds = None
         self._cupsconn.getPPDs2 (reply_handler=self._cups_getppds_reply,
                                  error_handler=self._cups_error)
 
@@ -117,24 +86,26 @@ class FetchedPPDs(gobject.GObject):
         self._ppds = cupshelpers.ppds.PPDs (result, language=self._language)
         self.emit ('ready')
 
-gobject.type_register (FetchedPPDs)
-
 class GetBestDriversRequest:
     def __init__ (self, device_id, device_make_and_model, device_uri,
                   cupsconn, language, reply_handler, error_handler):
         self.device_id = device_id
         self.device_make_and_model = device_make_and_model
         self.device_uri = device_uri
+        self.cupsconn = cupsconn
+        self.language = language
         self.reply_handler = reply_handler
         self.error_handler = error_handler
         self._signals = []
+        self.installed_files = []
+        self.download_tried = False
         debugprint ("+%s" % self)
 
         g_killtimer.add_hold ()
         global g_ppds
-        if g_ppds == None:
+        if g_ppds is None:
             debugprint ("GetBestDrivers request: need to fetch PPDs")
-            g_ppds = FetchedPPDs (cupsconn, language)
+            g_ppds = FetchedPPDs (self.cupsconn, self.language)
             self._signals.append (g_ppds.connect ('ready', self._ppds_ready))
             self._signals.append (g_ppds.connect ('error', self._ppds_error))
             g_ppds.run ()
@@ -161,6 +132,10 @@ class GetBestDriversRequest:
         self.error_handler (exc)
 
     def _ppds_ready (self, fetchedppds):
+        if not fetchedppds.is_ready ():
+            # PPDs being reloaded. Wait for next 'ready' signal.
+            return
+
         self._disconnect_signals ()
         ppds = fetchedppds.get_ppds ()
 
@@ -175,6 +150,7 @@ class GetBestDriversRequest:
                 id_dict["MDL"] = mdl
                 id_dict["DES"] = ""
                 id_dict["CMD"] = []
+                self.device_id = "MFG:%s;MDL:%s;" % (mfg, mdl)
 
             fit = ppds.getPPDNamesFromDeviceID (id_dict["MFG"],
                                                 id_dict["MDL"],
@@ -183,11 +159,42 @@ class GetBestDriversRequest:
                                                 self.device_uri,
                                                 self.device_make_and_model)
 
-            ppdnamelist = ppds.orderPPDNamesByPreference (fit.keys ())
+            ppdnamelist = ppds.orderPPDNamesByPreference (fit.keys (),
+                                                          self.installed_files,
+                                                          devid=id_dict,
+                                                          fit=fit)
+            ppdname = ppdnamelist[0]
+            status = fit[ppdname]
+
+            try:
+                if status != "exact" and not self.download_tried:
+                    self.download_tried = True
+                    self.dialog = newprinter.NewPrinterGUI()
+                    self.dialog.NewPrinterWindow.set_modal (False)
+                    self.handles = \
+                                   [self.dialog.connect ('dialog-canceled',
+                                                         self.on_dialog_canceled),
+                                    self.dialog.connect ('driver-download-checked',
+                                                         self.on_driver_download_checked)]
+
+                    self.reply_if_fail = [(x, fit[x]) for x in ppdnamelist]
+                    if not self.dialog.init ('download_driver',
+                                             devid=self.device_id):
+                        try:
+                            g_killtimer.remove_hold ()
+                        finally:
+                            e = RuntimeError ("Failed to launch dialog")
+                            self.error_handler (e)
+
+                    return
+            except:
+                # Ignore driver download if packages needed for the GUI are not
+                # installed or if no windows can be opened
+                pass
 
             g_killtimer.remove_hold ()
-            self.reply_handler (map (lambda x: (x, fit[x]), ppdnamelist))
-        except Exception, e:
+            self.reply_handler ([(x, fit[x]) for x in ppdnamelist])
+        except Exception as e:
             try:
                 g_killtimer.remove_hold ()
             except:
@@ -195,6 +202,30 @@ class GetBestDriversRequest:
 
             self.error_handler (e)
 
+    def _destroy_dialog (self):
+        for handle in self.handles:
+            self.dialog.disconnect (handle)
+
+        self.dialog.destroy ()
+        del self.dialog
+
+    def on_driver_download_checked(self, obj, installed_files):
+        if len (installed_files) > 0:
+            debugprint ("GetBestDrivers request: Re-fetch PPDs after driver download")
+            self._signals.append (g_ppds.connect ('ready', self._ppds_ready))
+            self._signals.append (g_ppds.connect ('error', self._ppds_error))
+            g_ppds.run ()
+            return
+
+        g_killtimer.remove_hold ()
+        self._destroy_dialog ()
+        self.reply_handler (self.reply_if_fail)
+
+    def on_dialog_canceled(self, obj):
+        g_killtimer.remove_hold ()
+        self._destroy_dialog ()
+        self.reply_handler (self.reply_if_fail)
+
 class GroupPhysicalDevicesRequest:
     def __init__ (self, devices, reply_handler, error_handler):
         self.devices = devices
@@ -206,7 +237,7 @@ class GroupPhysicalDevicesRequest:
             g_killtimer.add_hold ()
             need_resolving = {}
             self.deviceobjs = {}
-            for device_uri, device_dict in self.devices.iteritems ():
+            for device_uri, device_dict in self.devices.items ():
                 deviceobj = cupshelpers.Device (device_uri, **device_dict)
                 self.deviceobjs[device_uri] = deviceobj
                 if device_uri.startswith ("dnssd://"):
@@ -217,7 +248,7 @@ class GroupPhysicalDevicesRequest:
                 resolver.resolve (reply_handler=self._group)
             else:
                 self._group ()
-        except Exception, e:
+        except Exception as e:
             g_killtimer.remove_hold ()
             self.error_handler (e)
 
@@ -229,7 +260,7 @@ class GroupPhysicalDevicesRequest:
         # (in self.devices) have been modified.
         try:
             self.physdevs = []
-            for device_uri, deviceobj in self.deviceobjs.iteritems ():
+            for device_uri, deviceobj in self.deviceobjs.items ():
                 newphysicaldevice = PhysicalDevice.PhysicalDevice (deviceobj)
                 matched = False
                 try:
@@ -240,12 +271,11 @@ class GroupPhysicalDevicesRequest:
 
             uris_by_phys = []
             for physdev in self.physdevs:
-                uris_by_phys.append (map (lambda x: x.uri,
-                                          physdev.get_devices ()))
+                uris_by_phys.append ([x.uri for x in physdev.get_devices ()])
 
             g_killtimer.remove_hold ()
             self.reply_handler (uris_by_phys)
-        except Exception, e:
+        except Exception as e:
             g_killtimer.remove_hold ()
             self.error_handler (e)
 
@@ -260,12 +290,15 @@ class ConfigPrintingNewPrinterDialog(dbus.service.Object):
                         self.dialog.connect ('printer-added',
                                              self.on_printer_added),
                         self.dialog.connect ('printer-modified',
-                                             self.on_printer_modified)]
+                                             self.on_printer_modified),
+                        self.dialog.connect ('driver-download-checked',
+                                             self.on_driver_download_checked)]
         self._ppdcache = ppdcache.PPDCache ()
         self._cupsconn = cupsconn
         debugprint ("+%s" % self)
 
     def __del__ (self):
+        self.dialog.destroy ()
         debugprint ("-%s" % self)
 
     @dbus.service.method(dbus_interface=CONFIG_NEWPRINTERDIALOG_IFACE,
@@ -275,6 +308,12 @@ class ConfigPrintingNewPrinterDialog(dbus.service.Object):
         self.dialog.init ('printer_with_uri', device_uri=device_uri,
                           devid=device_id, xid=xid)
 
+    @dbus.service.method(dbus_interface=CONFIG_NEWPRINTERDIALOG_IFACE,
+                         in_signature='us', out_signature='')
+    def DownloadDriverForDeviceID(self, xid, device_id):
+        g_killtimer.add_hold ()
+        self.dialog.init ('download_driver', devid=device_id, xid=xid)
+
     @dbus.service.method(dbus_interface=CONFIG_NEWPRINTERDIALOG_IFACE,
                          in_signature='uss', out_signature='')
     def ChangePPD(self, xid, name, device_id):
@@ -313,24 +352,39 @@ class ConfigPrintingNewPrinterDialog(dbus.service.Object):
     def PrinterModified(self, name, ppd_has_changed):
         pass
 
+    @dbus.service.signal(dbus_interface=CONFIG_NEWPRINTERDIALOG_IFACE,
+                         signature='a(s)')
+    def DriverDownloadChecked(self, installed_files):
+        pass
+
     def on_dialog_canceled(self, obj):
+        debugprint ("%s: dialog canceled" % self)
         g_killtimer.remove_hold ()
         self.DialogCanceled ()
         self.remove_handles ()
         self.remove_from_connection ()
 
     def on_printer_added(self, obj, name):
+        debugprint ("%s: printer added" % self)
         g_killtimer.remove_hold ()
         self.PrinterAdded (name)
         self.remove_handles ()
         self.remove_from_connection ()
 
     def on_printer_modified(self, obj, name, ppd_has_changed):
+        debugprint ("%s: printer modified" % self)
         g_killtimer.remove_hold ()
         self.PrinterModifed (name, ppd_has_changed)
         self.remove_handles ()
         self.remove_from_connection ()
 
+    def on_driver_download_checked(self, obj, installed_files):
+        debugprint ("%s: driver download checked" % self)
+        g_killtimer.remove_hold ()
+        self.DriverDownloadChecked (installed_files)
+        self.remove_handles ()
+        self.remove_from_connection ()
+
     def remove_handles (self):
         for handle in self.handles:
             self.dialog.disconnect (handle)
@@ -369,8 +423,11 @@ class ConfigPrintingJobApplet(dbus.service.Object):
     def __init__ (self, bus, path):
         bus_name = dbus.service.BusName (CONFIG_BUS, bus=bus)
         dbus.service.Object.__init__ (self, bus_name=bus_name, object_path=path)
+        Gdk.threads_enter ()
         self.jobapplet = jobviewer.JobViewer(bus=dbus.SystemBus (),
                                              applet=True, my_jobs=True)
+        self.jobapplet.set_process_pending (False)
+        Gdk.threads_leave ()
         handle = self.jobapplet.connect ('finished', self.on_jobapplet_finished)
         self.finished_handle = handle
         self.has_finished = False
@@ -434,7 +491,7 @@ class ConfigPrinting(dbus.service.Object):
     @dbus.service.method(dbus_interface=CONFIG_IFACE,
                          in_signature='', out_signature='s')
     def JobApplet(self):
-       if self._jobapplet == None or self._jobapplet.has_finished:
+       if self._jobapplet is None or self._jobapplet.has_finished:
             self._pathn += 1
             path = "%s/JobApplet/%s" % (CONFIG_PATH, self._pathn)
             self._jobapplet = ConfigPrintingJobApplet (self.bus, path)
@@ -472,10 +529,10 @@ def _client_demo ():
             sys.argv[3] == '--devid'):
             device_id = sys.argv[4]
     else:
-        print "Device URI required"
+        print ("Device URI required")
         return
 
-    import gtk
+    from gi.repository import Gtk
     bus = dbus.SessionBus ()
     obj = bus.get_object (CONFIG_BUS, CONFIG_PATH)
     iface = dbus.Interface (obj, CONFIG_IFACE)
@@ -484,38 +541,37 @@ def _client_demo ():
 
     obj = bus.get_object (CONFIG_BUS, path)
     iface = dbus.Interface (obj, CONFIG_NEWPRINTERDIALOG_IFACE)
-    loop = gobject.MainLoop ()
+    loop = GObject.MainLoop ()
     def on_canceled(path=None):
-        print "%s: Dialog canceled" % path
+        print ("%s: Dialog canceled" % path)
         loop.quit ()
 
     def on_added(name, path=None):
-        print "%s: Printer '%s' added" % (path, name)
+        print ("%s: Printer '%s' added" % (path, name))
         loop.quit ()
 
-    w = gtk.Window ()
-    w.show_now ()
     iface.connect_to_signal ("DialogCanceled", on_canceled,
                              path_keyword="path")
     iface.connect_to_signal ("PrinterAdded", on_added,
                              path_keyword="path")
 
-    iface.NewPrinterFromDevice (w.window.xid, device_uri, device_id)
+    iface.NewPrinterFromDevice (0, device_uri, device_id)
     loop.run ()
 
 if __name__ == '__main__':
     import ppdippstr
+    import config
     import gettext
+    gettext.install(domain=config.PACKAGE, localedir=config.localedir)
+
     import locale
     try:
         locale.setlocale (locale.LC_ALL, "")
     except:
         pass
 
-    gettext.textdomain (config.PACKAGE)
-    gettext.bindtextdomain (config.PACKAGE, config.localedir)
     ppdippstr.init ()
-    gobject.threads_init ()
+    Gdk.threads_init ()
     from dbus.glib import DBusGMainLoop
     DBusGMainLoop (set_as_default=True)
 
@@ -524,6 +580,7 @@ if __name__ == '__main__':
         for opt in sys.argv[1:]:
             if opt == "--debug":
                 set_debugging (True)
+                cupshelpers.set_debugprint_fn (debugprint)
             elif opt == "--client":
                 client_demo = True
 
@@ -532,8 +589,9 @@ if __name__ == '__main__':
         sys.exit (0)
 
     debugprint ("Service running...")
-    loop = gobject.MainLoop ()
-    g_killtimer = KillTimer (killfunc=loop.quit)
+    g_killtimer = killtimer.KillTimer (killfunc=Gtk.main_quit)
     cp = ConfigPrinting ()
-    loop.run ()
+    Gdk.threads_enter ()
+    Gtk.main ()
+    Gdk.threads_leave ()
     cp.destroy ()
