@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## system-config-printer
 
-## Copyright (C) 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -21,11 +21,12 @@
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import config
-from gettext import gettext as _
+import gettext
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 import cups
 import dbus
-import gobject
-import gtk
+from gi.repository import GObject
+from gi.repository import Gtk
 import os
 import socket
 import tempfile
@@ -34,7 +35,7 @@ import time
 import authconn
 from debug import *
 from errordialogs import *
-import firewall
+import firewallsettings
 from gui import GtkGUI
 
 try:
@@ -44,10 +45,10 @@ except AttributeError:
     try_CUPS_SERVER_REMOTE_ANY = "_remote_any"
 
 # Set up "Problems?" link button
-class _UnobtrusiveButton(gtk.Button):
+class _UnobtrusiveButton(Gtk.Button):
     def __init__ (self, **args):
-        gtk.Button.__init__ (self, **args)
-        self.set_relief (gtk.RELIEF_NONE)
+        Gtk.Button.__init__ (self, **args)
+        self.set_relief (Gtk.ReliefStyle.NONE)
         label = self.get_child ()
         text = label.get_text ()
         label.set_use_markup (True)
@@ -58,15 +59,15 @@ class _UnobtrusiveButton(gtk.Button):
 class ServerSettings(GtkGUI):
 
     __gsignals__ = {
-        'settings-applied': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
-        'dialog-canceled': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
-        'problems-clicked': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
+        'settings-applied': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'dialog-canceled': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'problems-clicked': (GObject.SignalFlags.RUN_LAST, None, ()),
         }
 
     RESOURCE="/admin/conf/cupsd.conf"
 
     def __init__ (self, host=None, encryption=None, parent=None):
-        gobject.GObject.__init__ (self)
+        GObject.GObject.__init__ (self)
         self.cupsconn = authconn.Connection (host=host, encryption=encryption)
         self._host = host
         self._parent = parent
@@ -105,12 +106,12 @@ class ServerSettings(GtkGUI):
         self.remove = self.btAdvServerRemove
 
         selection = self.browse_treeview.get_selection ()
-        selection.set_mode (gtk.SELECTION_MULTIPLE)
+        selection.set_mode (Gtk.SelectionMode.MULTIPLE)
         self._connect (selection, 'changed', self.on_treeview_selection_changed)
 
         for column in self.browse_treeview.get_columns():
             self.browse_treeview.remove_column(column)
-        col = gtk.TreeViewColumn ('', gtk.CellRendererText (), text=0)
+        col = Gtk.TreeViewColumn ('', Gtk.CellRendererText (), text=0)
         self.browse_treeview.append_column (col)
 
         self._fillAdvanced ()
@@ -119,6 +120,7 @@ class ServerSettings(GtkGUI):
         if parent:
             self.dialog.set_transient_for (parent)
 
+        self.connect_signals ()
         self.dialog.show ()
 
     def get_dialog (self):
@@ -129,10 +131,11 @@ class ServerSettings(GtkGUI):
 
     def _fillAdvanced(self):
         # Fetch cupsd.conf
-        f = tempfile.TemporaryFile ()
+        f = tempfile.TemporaryFile () # mode='w+b'
         try:
             self.cupsconn.getFile (self.RESOURCE, file=f)
-        except cups.HTTPError, (s,):
+        except cups.HTTPError as e:
+            (s,) = e.args
             show_HTTP_Error (s, self._parent)
             raise
 
@@ -154,7 +157,8 @@ class ServerSettings(GtkGUI):
         browsing = True
         self.browse_poll = []
         f.seek (0)
-        for line in f.readlines ():
+        for line in f:
+            line = line.decode ('UTF-8')
             l = line.lower ().strip ()
             if l.startswith ("preservejobhistory "):
                 try:
@@ -186,7 +190,7 @@ class ServerSettings(GtkGUI):
         self.preserve_job_history = preserve_job_history
         self.preserve_job_files = preserve_job_files
 
-        model = gtk.ListStore (gobject.TYPE_STRING)
+        model = Gtk.ListStore (str)
         self.browse_treeview.set_model (model)
         for server in self.browse_poll:
             model.append (row=[server])
@@ -196,7 +200,8 @@ class ServerSettings(GtkGUI):
         self.cupsconn._begin_operation (_("fetching server settings"))
         try:
             self.server_settings = self.cupsconn.adminGetServerSettings()
-        except cups.IPPError, (e, m):
+        except cups.IPPError as e:
+            (e, m) = e.args
             show_IPP_Error(e, m, self._parent)
             self.cupsconn._end_operation ()
             raise
@@ -210,13 +215,20 @@ class ServerSettings(GtkGUI):
             (self.chkServerRemoteAdmin, cups.CUPS_SERVER_REMOTE_ADMIN),
             (self.chkServerAllowCancelAll, cups.CUPS_SERVER_USER_CANCEL_ANY),
             (self.chkServerLogDebug, cups.CUPS_SERVER_DEBUG_LOGGING),]:
-            widget.set_data("setting", setting)
-            if self.server_settings.has_key(setting):
+            widget.setting = setting
+            if setting in self.server_settings:
                 widget.set_active(int(self.server_settings[setting]))
                 widget.set_sensitive(True)
+                widget.show()
             else:
                 widget.set_active(False)
                 widget.set_sensitive(False)
+                widget.hide()
+
+        if cups.CUPS_SERVER_REMOTE_PRINTERS in self.server_settings:
+            self.frameBrowseServers.show()
+        else:
+            self.frameBrowseServers.hide()
 
         try:
             flag = cups.CUPS_SERVER_SHARE_PRINTERS
@@ -230,8 +242,8 @@ class ServerSettings(GtkGUI):
 
     def on_server_changed(self, widget):
         debugprint ("on_server_changed: %s" % widget)
-        setting = widget.get_data("setting")
-        if self.server_settings.has_key (setting):
+        setting = widget.setting
+        if setting in self.server_settings:
             if str(int(widget.get_active())) == self.server_settings[setting]:
                 self.changed.discard(widget)
             else:
@@ -239,16 +251,16 @@ class ServerSettings(GtkGUI):
 
         sharing = self.chkServerShare.get_active ()
         self.chkServerShareAny.set_sensitive (
-            sharing and self.server_settings.has_key(try_CUPS_SERVER_REMOTE_ANY))
+            sharing and try_CUPS_SERVER_REMOTE_ANY in self.server_settings)
 
     def _connect (self, widget, signal, handler, reason=None):
         id = widget.connect (signal, handler)
-        if not self.handler_ids.has_key (reason):
+        if reason not in self.handler_ids:
             self.handler_ids[reason] = []
         self.handler_ids[reason].append ((widget, id))
 
     def _disconnect (self, reason=None):
-        if self.handler_ids.has_key (reason):
+        if reason in self.handler_ids:
             for (widget, id) in self.handler_ids[reason]:
                 widget.disconnect (id)
             del self.handler_ids[reason]
@@ -261,9 +273,9 @@ class ServerSettings(GtkGUI):
         iter = model.insert (0, row=[_("Enter hostname")])
         button.set_sensitive (False)
         col = self.browse_treeview.get_columns ()[0]
-        cell = col.get_cell_renderers ()[0]
+        cell = col.get_cells ()[0]
         cell.set_property ('editable', True)
-        self.browse_treeview.set_cursor ((0,), col, start_editing=True)
+        self.browse_treeview.set_cursor (Gtk.TreePath(), col, True)
         self._connect (cell, 'edited', self.on_browse_poll_edited, 'edit')
         self._connect (cell, 'editing-canceled',
                        self.on_browse_poll_edit_cancel, 'edit')
@@ -272,7 +284,7 @@ class ServerSettings(GtkGUI):
         model = self.browse_treeview.get_model ()
         iter = model.get_iter (path)
         model.set_value (iter, 0, newvalue)
-        cell.stop_editing (canceled=False)
+        cell.stop_editing (False)
         cell.set_property ('editable', False)
         self.add.set_sensitive (True)
         self._disconnect ('edit')
@@ -325,10 +337,10 @@ class ServerSettings(GtkGUI):
             model.remove (iter)
 
     def on_browse_poll_edit_cancel (self, cell):
-        cell.stop_editing (canceled=True)
+        cell.stop_editing (True)
         cell.set_property ('editable', False)
         model = self.browse_treeview.get_model ()
-        iter = model.get_iter ((0,))
+        iter = model.get_iter (Gtk.TreePath())
         model.remove (iter)
         self.add.set_sensitive (True)
         self.remove.set_sensitive (False)
@@ -338,16 +350,15 @@ class ServerSettings(GtkGUI):
         model = self.browse_treeview.get_model ()
         selection = self.browse_treeview.get_selection ()
         rows = selection.get_selected_rows ()
-        refs = map (lambda path: gtk.TreeRowReference (model, path),
-                    rows[1])
+        refs = [Gtk.TreeRowReference.new (model, path) for path in rows[1]]
         for ref in refs:
             path = ref.get_path ()
             iter = model.get_iter (path)
             model.remove (iter)
 
     def on_response (self, dialog, response):
-        if (response == gtk.RESPONSE_CANCEL or
-            response != gtk.RESPONSE_OK):
+        if (response == Gtk.ResponseType.CANCEL or
+            response != Gtk.ResponseType.OK):
             self._disconnect ()
             self.dialog.hide ()
             self.emit ('dialog-canceled')
@@ -396,10 +407,11 @@ class ServerSettings(GtkGUI):
             return
 
         # Fetch cupsd.conf afresh
-        f = tempfile.TemporaryFile ()
+        f = tempfile.TemporaryFile () # mode='w+b'
         try:
             self.cupsconn.getFile (self.RESOURCE, file=f)
-        except cups.HTTPError, (s,):
+        except cups.HTTPError as e:
+            (s,) = e.args
             show_HTTP_Error (s, self.dialog)
             return
 
@@ -417,18 +429,19 @@ class ServerSettings(GtkGUI):
             browsepoll_lines += "BrowsePoll %s\n" % server
 
         f.seek (0)
-        conf = tempfile.TemporaryFile ()
+        conf = tempfile.TemporaryFile () # mode='w+b'
         wrote_preserve_history = wrote_preserve_files = False
         wrote_browsepoll = False
         has_browsepoll = False
-        lines = f.readlines ()
-        for line in lines:
+        for line in f:
+            line = line.decode('UTF-8')
             l = line.lower ().strip ()
             if l.startswith ("browsepoll "):
                 has_browsepoll = True
                 break
 
-        for line in lines:
+        for line in f:
+            line = line.decode('UTF-8')
             l = line.lower ().strip ()
             if l.startswith ("preservejobhistory "):
                 if wrote_preserve_history:
@@ -450,7 +463,7 @@ class ServerSettings(GtkGUI):
                     # Ignore extra BrowsePoll lines.
                     continue
                 # Write new BrowsePoll section.
-                conf.write (browsepoll_lines)
+                conf.write (browsepoll_lines.encode('UTF-8'))
                 wrote_browsepoll = True
                 # Don't write out the original BrowsePoll line.
                 continue
@@ -458,27 +471,32 @@ class ServerSettings(GtkGUI):
                   l.startswith ("browsing ")):
                 if not wrote_browsepoll:
                     # Write original Browsing line.
-                    conf.write (line)
+                    conf.write (line.encode('UTF-8'))
                     # Write new BrowsePoll section.
-                    conf.write (browsepoll_lines)
+                    conf.write (browsepoll_lines.encode('UTF-8'))
                     wrote_browsepoll = True
                     continue
 
-            conf.write (line)
+            conf.write (line.encode('UTF-8'))
 
         if not wrote_preserve_history:
-            conf.write (job_history_line)
+            conf.write (job_history_line.encode('UTF-8'))
         if not wrote_preserve_files:
-            conf.write (job_files_line)
+            conf.write (job_files_line.encode('UTF-8'))
         if not wrote_browsepoll:
-            conf.write (browsepoll_lines)
+            conf.write (browsepoll_lines.encode('UTF-8'))
 
         conf.flush ()
         fd = conf.fileno ()
         os.lseek (fd, 0, os.SEEK_SET)
         try:
             self.cupsconn.putFile ("/admin/conf/cupsd.conf", fd=fd)
-        except cups.HTTPError, (s,):
+        except cups.IPPError as e:
+            (e, m) = e.args
+            show_IPP_Error (e, m, self.dialog)
+            return
+        except cups.HTTPError as e:
+            (s,) = e.args
             show_HTTP_Error (s, self.dialog)
             return
 
@@ -501,16 +519,17 @@ class ServerSettings(GtkGUI):
             (self.chkServerRemoteAdmin, cups.CUPS_SERVER_REMOTE_ADMIN),
             (self.chkServerAllowCancelAll, cups.CUPS_SERVER_USER_CANCEL_ANY),
             (self.chkServerLogDebug, cups.CUPS_SERVER_DEBUG_LOGGING),]:
-            if not self.server_settings.has_key(setting): continue
+            if setting not in self.server_settings: continue
             setting_dict[setting] = str(int(widget.get_active()))
         self.cupsconn._begin_operation (_("modifying server settings"))
         try:
             self.cupsconn.adminSetServerSettings(setting_dict)
-        except cups.IPPError, (e, m):
+        except cups.IPPError as e:
+            (e, m) = e.args
             show_IPP_Error(e, m, self.dialog)
             self.cupsconn._end_operation ()
             return True
-        except RuntimeError, s:
+        except RuntimeError as s:
             show_IPP_Error(None, s, self.dialog)
             self.cupsconn._end_operation ()
             return True
@@ -526,7 +545,10 @@ class ServerSettings(GtkGUI):
             try:
                 if (self._host == 'localhost' or
                     self._host[0] == '/'):
-                    f = firewall.Firewall ()
+                    f = firewallsettings.FirewallD ()
+                    if not f.running:
+                        f = firewallsettings.SystemConfigFirewall ()
+
                     allowed = f.check_ipp_server_allowed ()
                 else:
                     # This is a remote server.  Nothing we can do
@@ -534,22 +556,21 @@ class ServerSettings(GtkGUI):
                     allowed = True
 
                 if not allowed:
-                    dialog = gtk.MessageDialog (self.ServerSettingsDialog,
-                                                gtk.DIALOG_MODAL |
-                                                gtk.DIALOG_DESTROY_WITH_PARENT,
-                                                gtk.MESSAGE_QUESTION,
-                                                gtk.BUTTONS_NONE,
-                                                _("Adjust Firewall"))
+                    dialog = Gtk.MessageDialog (parent=self.ServerSettingsDialog,
+                                                modal=True, destroy_with_parent=True,
+                                                message_type=Gtk.MessageType.QUESTION,
+                                                buttons=Gtk.ButtonsType.NONE,
+                                                text=_("Adjust Firewall"))
                     dialog.format_secondary_text (_("Adjust the firewall now "
                                                     "to allow all incoming IPP "
                                                     "connections?"))
-                    dialog.add_buttons (gtk.STOCK_CANCEL, gtk.RESPONSE_NO,
-                                        _("Adjust Firewall"), gtk.RESPONSE_YES)
+                    dialog.add_buttons (Gtk.STOCK_CANCEL, Gtk.ResponseType.NO,
+                                        _("Adjust Firewall"), Gtk.ResponseType.YES)
                     response = dialog.run ()
                     dialog.destroy ()
 
-                    if response == gtk.RESPONSE_YES:
-                        f.add_rule (f.ALLOW_IPP_SERVER)
+                    if response == Gtk.ResponseType.YES:
+                        f.add_service (firewallsettings.IPP_SERVER_SERVICE)
                         f.write ()
             except (dbus.DBusException, Exception):
                 nonfatalException ()
@@ -559,17 +580,15 @@ class ServerSettings(GtkGUI):
         # Now reconnect, in case the server needed to reload.
         self._reconnect ()
 
-gobject.type_register (ServerSettings)
-
 if __name__ == '__main__':
     os.environ['SYSTEM_CONFIG_PRINTER_UI'] = 'ui'
-    loop = gobject.MainLoop ()
+    loop = GObject.MainLoop ()
 
     def quit (*args):
         loop.quit ()
 
     def problems (obj):
-        print "%s: problems" % obj
+        print("%s: problems" % obj)
 
     set_debugging (True)
     s = ServerSettings ()
