@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2008, 2009, 2010, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -20,29 +20,32 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+from gi.repository import Gtk
+
 import cups
 import cupshelpers
 import installpackage
 import os
 import subprocess
 from timedops import TimedOperation, TimedSubprocess
-from base import *
+from .base import *
+from functools import reduce
 
 class CheckPPDSanity(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Check PPD sanity")
-        vbox = gtk.VBox ()
+        vbox = Gtk.VBox ()
         vbox.set_border_width (12)
         vbox.set_spacing (12)
-        self.label = gtk.Label ()
+        self.label = Gtk.Label ()
         self.label.set_line_wrap (True)
         self.label.set_use_markup (True)
         self.label.set_alignment (0, 0)
         vbox.pack_start (self.label, False, False, 0)
 
-        box = gtk.HButtonBox ()
-        box.set_layout (gtk.BUTTONBOX_START)
-        self.install_button = gtk.Button (_("Install"))
+        box = Gtk.HButtonBox ()
+        box.set_layout (Gtk.ButtonBoxStyle.START)
+        self.install_button = Gtk.Button.new_with_label (_("Install"))
         box.add (self.install_button)
         # Although we want this hidden initially,
         # troubleshooter.new_page will call show_all() on the widget
@@ -100,7 +103,7 @@ class CheckPPDSanity(Question):
                 self.op = TimedSubprocess (parent=parent,
                                            args=['cupstestppd', '-rvv', tmpf],
                                            close_fds=True,
-                                           stdin=file("/dev/null"),
+                                           stdin=subprocess.DEVNULL,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
                 result = self.op.run ()
@@ -117,7 +120,7 @@ class CheckPPDSanity(Question):
         if tmpf:
             os.unlink (tmpf)
 
-        if title == None and not answers['cups_printer_remote']:
+        if title is None and not answers['cups_printer_remote']:
             (pkgs, exes) = cupshelpers.missingPackagesAndExecutables (ppd)
             self.answers['missing_pkgs_and_exes'] = (pkgs, exes)
             if len (pkgs) > 0 or len (exes) > 0:
@@ -139,11 +142,11 @@ class CheckPPDSanity(Question):
                              "is not currently installed.") % (name,
                                                                (exes + pkgs)[0])
 
-        if title != None:
+        if title is not None:
             self.label.set_markup ('<span weight="bold" size="larger">' +
                                    title + '</span>\n\n' + text)
 
-        return title != None
+        return title is not None
 
     def connect_signals (self, handle):
         self.button_sigid = self.install_button.connect ("clicked",
