@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
 ## Printing troubleshooter
 
-## Copyright (C) 2010 Red Hat, Inc.
+## Copyright (C) 2010, 2014 Red Hat, Inc.
 ## Copyright (C) 2010 Jiri Popelka <jpopelka@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,10 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+from gi.repository import Gtk
+
 import subprocess
-from base import *
+from .base import *
 import os
 import shlex
 from timedops import TimedSubprocess
@@ -28,7 +30,7 @@ from timedops import TimedSubprocess
 class CheckSELinux(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Check SELinux contexts")
-        troubleshooter.new_page (gtk.Label (), self)
+        troubleshooter.new_page (Gtk.Label (), self)
 
     def display (self):
         self.answers = {}
@@ -45,8 +47,7 @@ class CheckSELinux(Question):
         if not selinux.is_selinux_enabled():
             return False
 
-        paths = ["/etc/cups/", "/usr/local/libexec/cups/", "/usr/local/share/cups/"]
-        null = file ("/dev/null", "r+")
+        paths = ["/etc/cups/", "/usr/lib/cups/", "/usr/share/cups/"]
         parent = self.troubleshooter.get_window ()
         contexts = {}
         new_environ = os.environ.copy()
@@ -58,9 +59,9 @@ class CheckSELinux(Question):
                                        args=restorecon_args,
                                        close_fds=True,
                                        env=new_environ,
-                                       stdin=null,
+                                       stdin=subprocess.DEVNULL,
                                        stdout=subprocess.PIPE,
-                                       stderr=null)
+                                       stderr=subprocess.DEVNULL)
             (restorecon_stdout, restorecon_stderr, result) = self.op.run ()
         except:
             # Problem executing command.
