@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2008, 2009, 2010, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -24,12 +24,14 @@ import glob
 import os
 import subprocess
 from timedops import TimedSubprocess
-import urllib
-from base import *
+import urllib.parse
+from .base import *
+from gi.repository import Gtk
+
 class CheckUSBPermissions(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Check USB permissions")
-        troubleshooter.new_page (gtk.Label (), self)
+        troubleshooter.new_page (Gtk.Label (), self)
 
     def display (self):
         self.answers = {}
@@ -45,7 +47,7 @@ class CheckUSBPermissions(Question):
         else:
             return False
 
-        (scheme, rest) = urllib.splittype (device_uri)
+        (scheme, rest) = urllib.parse.splittype (device_uri)
         if scheme not in ['hp', 'hpfax', 'usb', 'hal']:
             return False
 
@@ -67,7 +69,7 @@ class CheckUSBPermissions(Question):
                                        args=[LSUSB, "-v"],
                                        close_fds=True,
                                        env=new_environ,
-                                       stdin=file("/dev/null"),
+                                       stdin=subprocess.DEVNULL,
                                        stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE)
             (lsusb_stdout, lsusb_stderr, result) = self.op.run ()
@@ -79,7 +81,7 @@ class CheckUSBPermissions(Question):
         dev_by_id = {}
         this_dev = None
         for line in lsusb_stdout:
-            if (this_dev != None and
+            if (this_dev is not None and
                 ((line.find ("bInterfaceClass") != -1 and
                   line.find ("7 Printer") != -1) or
                  (line.find ("bInterfaceSubClass") != -1 and
@@ -133,8 +135,8 @@ class CheckUSBPermissions(Question):
         paths = []
         if not scheme.startswith ('hp'):
             paths.extend (glob.glob ("/dev/usb/lp?"))
-        for mfr_id, mdls in dev_by_id.iteritems ():
-            for mdl_id, devs in mdls.iteritems ():
+        for mfr_id, mdls in dev_by_id.items ():
+            for mdl_id, devs in mdls.items ():
                 for dev in devs:
                     path = "/dev/bus/usb/%s/%s" % (dev['bus'], dev['dev'])
                     paths.append (path)
@@ -147,11 +149,11 @@ class CheckUSBPermissions(Question):
                                            args=[GETFACL, path],
                                            close_fds=True,
                                            env=new_environ,
-                                           stdin=file("/dev/null"),
+                                           stdin=subprocess.DEVNULL,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
                 (getfacl_stdout, getfacl_stderr, result) = self.op.run ()
-                output = filter (lambda x: len (x) > 0, getfacl_stdout)
+                output = [x for x in getfacl_stdout if len (x) > 0]
             except:
                 # Problem executing command.
                 output = []
