@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008, 2009, 2011 Red Hat, Inc.
+## Copyright (C) 2008, 2009, 2011, 2012 Red Hat, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -19,10 +19,12 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+from gi.repository import Gtk
+
 import cups
-import gobject
+from gi.repository import GObject
 from timedops import TimedOperation
-from base import *
+from .base import *
 class ChooseNetworkPrinter(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Choose network printer")
@@ -31,13 +33,13 @@ class ChooseNetworkPrinter(Question):
                                      "are trying to use from the list below. "
                                      "If it does not appear in the list, "
                                      "select 'Not listed'."))
-        tv = gtk.TreeView ()
-        name = gtk.TreeViewColumn (_("Name"),
-                                   gtk.CellRendererText (), text=0)
-        location = gtk.TreeViewColumn (_("Location"),
-                                       gtk.CellRendererText (), text=1)
-        info = gtk.TreeViewColumn (_("Information"),
-                                   gtk.CellRendererText (), text=2)
+        tv = Gtk.TreeView ()
+        name = Gtk.TreeViewColumn (_("Name"),
+                                   Gtk.CellRendererText (), text=0)
+        location = Gtk.TreeViewColumn (_("Location"),
+                                       Gtk.CellRendererText (), text=1)
+        info = Gtk.TreeViewColumn (_("Information"),
+                                   Gtk.CellRendererText (), text=2)
         name.set_property ("resizable", True)
         location.set_property ("resizable", True)
         info.set_property ("resizable", True)
@@ -45,9 +47,9 @@ class ChooseNetworkPrinter(Question):
         tv.append_column (location)
         tv.append_column (info)
         tv.set_rules_hint (True)
-        sw = gtk.ScrolledWindow ()
-        sw.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        sw.set_shadow_type (gtk.SHADOW_IN)
+        sw = Gtk.ScrolledWindow ()
+        sw.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        sw.set_shadow_type (Gtk.ShadowType.IN)
         sw.add (tv)
         page1.pack_start (sw, True, True, 0)
         self.treeview = tv
@@ -63,14 +65,14 @@ class ChooseNetworkPrinter(Question):
 
         server = answers['remote_server_try_connect']
 
-        model = gtk.ListStore (gobject.TYPE_STRING,
-                               gobject.TYPE_STRING,
-                               gobject.TYPE_STRING,
-                               gobject.TYPE_PYOBJECT)
+        model = Gtk.ListStore (str,
+                               str,
+                               str,
+                               GObject.TYPE_PYOBJECT)
         self.model = model
         self.treeview.set_model (model)
         iter = model.append (None)
-        model.set (iter, 0, _("Not listed"), 1, '', 2, '', 3, None)
+        model.set (iter, 0, _("Not listed"), 1, '', 2, '', 3, 0)
 
         parent = self.troubleshooter.get_window ()
 
@@ -83,20 +85,20 @@ class ChooseNetworkPrinter(Question):
             dests = self.op.run ()
             printers = None
             dests_list = []
-            for (name, instance), dest in dests.iteritems ():
-                if name == None:
+            for (name, instance), dest in dests.items ():
+                if name is None:
                     continue
 
-                if instance != None:
+                if instance is not None:
                     queue = "%s/%s" % (name, instance)
                 else:
                     queue = name
 
-                if printers == None:
+                if printers is None:
                     self.op = TimedOperation (c.getPrinters)
                     printers = self.op.run ()
 
-                if not printers.has_key (name):
+                if name not in printers:
                     info = _("Unknown")
                     location = _("Unknown")
                 else:
@@ -106,7 +108,7 @@ class ChooseNetworkPrinter(Question):
 
                 dests_list.append ((queue, location, info, dest))
 
-            dests_list.sort (lambda x, y: cmp (x[0], y[0]))
+            dests_list.sort (key=lambda x: x[0])
             for queue, location, info, dest in dests_list:
                 iter = model.append (None)
                 model.set (iter, 0, queue, 1, location, 2, info, 3, dest)
@@ -128,7 +130,7 @@ class ChooseNetworkPrinter(Question):
 
     def can_click_forward (self):
         model, iter = self.treeview.get_selection ().get_selected ()
-        if iter == None:
+        if iter is None:
             return False
         return True
 
@@ -141,7 +143,7 @@ class ChooseNetworkPrinter(Question):
             return {}
 
         dest = model.get_value (iter, 3)
-        if dest == None:
+        if dest == 0:
             class enum_dests:
                 def __init__ (self, model):
                     self.dests = []
