@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.6
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2008, 2009, 2010, 2012 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -23,14 +23,16 @@
 import cups
 import dbus
 import dbus.glib
-import gobject
+from gi.repository import GLib
 import os
-import pango
+from gi.repository import Gdk
+from gi.repository import Gtk
+from gi.repository import Pango
 import tempfile
 import time
 from timedops import TimedOperation, OperationCanceled
 
-from base import *
+from .base import *
 
 import errordialogs
 from errordialogs import *
@@ -48,11 +50,11 @@ class PrintTestPage(Question):
 
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Print test page")
-        page = gtk.VBox ()
+        page = Gtk.VBox ()
         page.set_spacing (12)
         page.set_border_width (12)
 
-        label = gtk.Label ()
+        label = Gtk.Label ()
         label.set_alignment (0, 0)
         label.set_use_markup (True)
         label.set_line_wrap (True)
@@ -65,34 +67,34 @@ class PrintTestPage(Question):
                                   "print that document now and mark the print "
                                   "job below."))
 
-        hbox = gtk.HButtonBox ()
+        hbox = Gtk.HButtonBox ()
         hbox.set_border_width (0)
         hbox.set_spacing (3)
-        hbox.set_layout (gtk.BUTTONBOX_START)
-        self.print_button = gtk.Button (_("Print Test Page"))
+        hbox.set_layout (Gtk.ButtonBoxStyle.START)
+        self.print_button = Gtk.Button.new_with_label (_("Print Test Page"))
         hbox.pack_start (self.print_button, False, False, 0)
 
-        self.cancel_button = gtk.Button (_("Cancel All Jobs"))
+        self.cancel_button = Gtk.Button.new_with_label (_("Cancel All Jobs"))
         hbox.pack_start (self.cancel_button, False, False, 0)
         page.pack_start (hbox, False, False, 0)
 
-        tv = gtk.TreeView ()
-        test_cell = gtk.CellRendererToggle ()
-        test = gtk.TreeViewColumn (_("Test"), test_cell, active=0)
-        job = gtk.TreeViewColumn (_("Job"), gtk.CellRendererText (), text=1)
-        printer_cell = gtk.CellRendererText ()
-        printer = gtk.TreeViewColumn (_("Printer"), printer_cell, text=2)
-        name_cell = gtk.CellRendererText ()
-        name = gtk.TreeViewColumn (_("Document"), name_cell, text=3)
-        status = gtk.TreeViewColumn (_("Status"), gtk.CellRendererText (),
+        tv = Gtk.TreeView ()
+        test_cell = Gtk.CellRendererToggle ()
+        test = Gtk.TreeViewColumn (_("Test"), test_cell, active=0)
+        job = Gtk.TreeViewColumn (_("Job"), Gtk.CellRendererText (), text=1)
+        printer_cell = Gtk.CellRendererText ()
+        printer = Gtk.TreeViewColumn (_("Printer"), printer_cell, text=2)
+        name_cell = Gtk.CellRendererText ()
+        name = Gtk.TreeViewColumn (_("Document"), name_cell, text=3)
+        status = Gtk.TreeViewColumn (_("Status"), Gtk.CellRendererText (),
                                      text=4)
         test_cell.set_radio (False)
         self.test_cell = test_cell
         printer.set_resizable (True)
-        printer_cell.set_property ("ellipsize", pango.ELLIPSIZE_END)
+        printer_cell.set_property ("ellipsize", Pango.EllipsizeMode.END)
         printer_cell.set_property ("width-chars", 20)
         name.set_resizable (True)
-        name_cell.set_property ("ellipsize", pango.ELLIPSIZE_END)
+        name_cell.set_property ("ellipsize", Pango.EllipsizeMode.END)
         name_cell.set_property ("width-chars", 20)
         status.set_resizable (True)
         tv.append_column (test)
@@ -101,23 +103,22 @@ class PrintTestPage(Question):
         tv.append_column (name)
         tv.append_column (status)
         tv.set_rules_hint (True)
-        sw = gtk.ScrolledWindow ()
-        sw.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        sw.set_shadow_type (gtk.SHADOW_IN)
+        sw = Gtk.ScrolledWindow ()
+        sw.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        sw.set_shadow_type (Gtk.ShadowType.IN)
         sw.add (tv)
         self.treeview = tv
-        page.pack_start (sw)
+        page.pack_start (sw, False, False, 0)
 
-        label = gtk.Label (_("Did the marked print jobs print correctly?"))
+        label = Gtk.Label(label=_("Did the marked print jobs print correctly?"))
         label.set_line_wrap (True)
         label.set_alignment (0, 0)
         page.pack_start (label, False, False, 0)
 
-        vbox = gtk.VBox ()
+        vbox = Gtk.VBox ()
         vbox.set_spacing (6)
-        self.yes = gtk.RadioButton (label=_("Yes"))
-        no = gtk.RadioButton (label=_("No"))
-        no.set_group (self.yes)
+        self.yes = Gtk.RadioButton (label=_("Yes"))
+        no = Gtk.RadioButton.new_with_label_from_widget (self.yes, _("No"))
         vbox.pack_start (self.yes, False, False, 0)
         vbox.pack_start (no, False, False, 0)
         page.pack_start (vbox, False, False, 0)
@@ -126,7 +127,7 @@ class PrintTestPage(Question):
 
     def display (self):
         answers = self.troubleshooter.answers
-        if not answers.has_key ('cups_queue'):
+        if 'cups_queue' not in answers:
             return False
 
         parent = self.troubleshooter.get_window ()
@@ -134,12 +135,12 @@ class PrintTestPage(Question):
         mediatype = None
         defaults = answers.get ('cups_printer_ppd_defaults', {})
         for opts in defaults.values ():
-            for opt, value in opts.iteritems ():
+            for opt, value in opts.items ():
                 if opt == "MediaType":
                     mediatype = value
                     break
 
-        if mediatype != None:
+        if mediatype is not None:
             mediatype_string = '\n\n' + (_("Remember to load paper of type "
                                            "'%s' into the printer first.") %
                                          mediatype)
@@ -149,11 +150,11 @@ class PrintTestPage(Question):
         label_text = self.main_label_text + mediatype_string
         self.main_label.set_markup (label_text)
 
-        model = gtk.ListStore (gobject.TYPE_BOOLEAN,
-                               gobject.TYPE_INT,
-                               gobject.TYPE_STRING,
-                               gobject.TYPE_STRING,
-                               gobject.TYPE_STRING)
+        model = Gtk.ListStore (bool,
+                               int,
+                               str,
+                               str,
+                               str)
         self.treeview.set_model (model)
         self.job_to_iter = {}
 
@@ -187,10 +188,9 @@ class PrintTestPage(Question):
         # We want to display the jobs in the queue for this printer...
         try:
             queue_uri_ending = "/" + self.troubleshooter.answers['cups_queue']
-            jobs_on_this_printer = filter (lambda x:
-                                               jobs_dict[x]['job-printer-uri'].\
-                                               endswith (queue_uri_ending),
-                                           jobs_dict.keys ())
+            jobs_on_this_printer = [x for x in jobs_dict.keys () if \
+                                            jobs_dict[x]['job-printer-uri']. \
+                                            endswith (queue_uri_ending)]
         except:
             jobs_on_this_printer = []
 
@@ -251,7 +251,7 @@ class PrintTestPage(Question):
                                      path=DBUS_PATH,
                                      dbus_interface=DBUS_IFACE)
 
-        self.timer = gobject.timeout_add_seconds (1, self.update_jobs_list)
+        self.timer = GLib.timeout_add_seconds (1, self.update_jobs_list)
 
     def disconnect_signals (self):
         if self.bus:
@@ -281,7 +281,7 @@ class PrintTestPage(Question):
         except:
             pass
 
-        gobject.source_remove (self.timer)
+        GLib.source_remove (self.timer)
 
     def collect_answer (self):
         if not self.displayed:
@@ -313,7 +313,7 @@ class PrintTestPage(Question):
                         attrs = c.getJobAttributes (jobid)
                     except AttributeError:
                         # getJobAttributes was introduced in pycups 1.9.35.
-                        if job_attrs == None:
+                        if job_attrs is None:
                             job_attrs = c.getJobs (which_jobs='all')
 
                         attrs = self.job_attrs[jobid]
@@ -344,8 +344,8 @@ class PrintTestPage(Question):
 
     def handle_dbus_signal (self, *args):
         debugprint ("D-Bus signal caught: updating jobs list soon")
-        gobject.source_remove (self.timer)
-        self.timer = gobject.timeout_add (200, self.update_jobs_list)
+        GLib.source_remove (self.timer)
+        self.timer = GLib.timeout_add (200, self.update_jobs_list)
 
     def update_job (self, jobid, job_dict):
         iter = self.job_to_iter[jobid]
@@ -360,7 +360,7 @@ class PrintTestPage(Question):
             except KeyError:
                 printer_name = None
 
-        if printer_name != None:
+        if printer_name is not None:
             model.set_value (iter, 2, printer_name)
 
         model.set_value (iter, 3, job_dict['job-name'])
@@ -383,7 +383,7 @@ class PrintTestPage(Question):
         mimetypes = [None, 'text/plain']
         for mimetype in mimetypes:
             try:
-                if mimetype == None:
+                if mimetype is None:
                     # Default test page.
                     self.op = TimedOperation (print_test_page,
                                               (answers['cups_queue'],),
@@ -391,7 +391,7 @@ class PrintTestPage(Question):
                     jobid = self.op.run ()
                 elif mimetype == 'text/plain':
                     (tmpfd, tmpfname) = tempfile.mkstemp ()
-                    os.write (tmpfd, "This is a test page.\n")
+                    os.write (tmpfd, b"This is a test page.\n")
                     os.close (tmpfd)
                     self.op = TimedOperation (print_test_page,
                                               (answers['cups_queue'],),
@@ -416,11 +416,12 @@ class PrintTestPage(Question):
             except RuntimeError:
                 self.persistent_answers['test_page_submit_failure'] = 'connect'
                 break
-            except cups.IPPError, (e, s):
+            except cups.IPPError as e:
+                (e, s) = e.args
                 if (e == cups.IPP_DOCUMENT_FORMAT and
                     mimetypes.index (mimetype) < (len (mimetypes) - 1)):
                     # Try next format.
-                    if tmpfname != None:
+                    if tmpfname is not None:
                         os.unlink (tmpfname)
                         tmpfname = None
                     continue
@@ -435,7 +436,7 @@ class PrintTestPage(Question):
     def cancel_clicked (self, widget):
         self.persistent_answers['test_page_jobs_cancelled'] = True
         jobids = []
-        for jobid, iter in self.job_to_iter.iteritems ():
+        for jobid, iter in self.job_to_iter.items ():
             jobids.append (jobid)
 
         def cancel_jobs (jobids):
@@ -443,7 +444,8 @@ class PrintTestPage(Question):
             for jobid in jobids:
                 try:
                     c.cancelJob (jobid)
-                except cups.IPPError, (e, s):
+                except cups.IPPError as e:
+                    (e, s) = e.args
                     if e != cups.IPP_NOT_POSSIBLE:
                         self.persistent_answers['test_page_cancel_failure'] = (e, s)
 
@@ -474,7 +476,7 @@ class PrintTestPage(Question):
 
         # Enter the GDK lock.  We need to do this because we were
         # called from a timeout.
-        gtk.gdk.threads_enter ()
+        Gdk.threads_enter ()
 
         parent = self.troubleshooter.get_window ()
         self.op = TimedOperation (get_notifications,
@@ -483,7 +485,7 @@ class PrintTestPage(Question):
         try:
             notifications = self.op.run ()
         except (OperationCanceled, cups.IPPError):
-            gtk.gdk.threads_leave ()
+            Gdk.threads_leave ()
             return True
 
         answers = self.troubleshooter.answers
@@ -492,13 +494,8 @@ class PrintTestPage(Question):
         test_jobs = self.persistent_answers.get('test_page_job_id', [])
         for event in notifications['events']:
             seq = event['notify-sequence-number']
-            try:
-                if seq <= self.sub_seq:
-                    # Work around a bug in pycups < 1.9.34
-                    continue
-            except AttributeError:
-                pass
             self.sub_seq = seq
+
             job = event['notify-job-id']
 
             nse = event['notify-subscribed-event']
@@ -511,7 +508,7 @@ class PrintTestPage(Question):
                     model.set_value (iter, 1, job)
                 else:
                     continue
-            elif not self.job_to_iter.has_key (job):
+            elif job not in self.job_to_iter:
                 continue
 
             if (job in test_jobs and
@@ -524,11 +521,11 @@ class PrintTestPage(Question):
 
         # Update again when we're told to. (But we might update sooner if
         # there is a D-Bus signal.)
-        gobject.source_remove (self.timer)
-        self.timer = gobject.timeout_add_seconds (
+        GLib.source_remove (self.timer)
+        self.timer = GLib.timeout_add_seconds (
             notifications['notify-get-interval'],
             self.update_jobs_list)
         debugprint ("Update again in %ds" %
                     notifications['notify-get-interval'])
-        gtk.gdk.threads_leave ()
+        Gdk.threads_leave ()
         return False
