@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## Printing troubleshooter
 
@@ -20,18 +20,21 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+from gi.repository import Gtk
+
 import cups
 import ppdcache
 import statereason
 from timedops import TimedOperation
-from base import *
+from .base import *
+from functools import reduce
 class PrinterStateReasons(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Printer state reasons")
         page = self.initial_vbox (_("Status Messages"),
                                   _("There are status messages associated with "
                                     "this queue."))
-        self.label = gtk.Label ()
+        self.label = Gtk.Label ()
         self.label.set_alignment (0, 0)
         self.label.set_line_wrap (True)
         page.pack_start (self.label, False, False, 0)
@@ -63,7 +66,7 @@ class PrinterStateReasons(Question):
             text += '\n\n'
 
         state_reasons_list = dict['printer-state-reasons']
-        if type (state_reasons_list) == unicode:
+        if type (state_reasons_list) == str:
             state_reasons_list = [state_reasons_list]
 
         self.state_message = state_message
@@ -100,7 +103,7 @@ class PrinterStateReasons(Question):
 
         # If this screen has been show before, don't show it again if
         # nothing changed.
-        if troubleshooter.answers.has_key ('printer-state-message'):
+        if 'printer-state-message' in troubleshooter.answers:
             if (troubleshooter.answers['printer-state-message'] ==
                 self.state_message and
                 troubleshooter.answers['printer-state-reasons'] ==
