@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## Printing troubleshooter
 
@@ -21,15 +21,16 @@
 
 import cups
 from timedops import TimedOperation
-from base import *
+from .base import *
 class SchedulerNotRunning(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Scheduler not running?")
         page = self.initial_vbox (_("CUPS Service Stopped"),
                                   _("The CUPS print spooler does not appear "
-                                    "to be running.  To correct this, run "
-                                    "'/etc/rc.d/cups start' "
-                                    "as root in a terminal."))
+                                    "to be running.  To correct this, choose "
+                                    "System->Administration->Services from "
+                                    "the main menu and look for the 'cups' "
+                                    "service."))
         troubleshooter.new_page (page, self)
 
     def display (self):
