@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## Printing troubleshooter
 
@@ -19,14 +19,16 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-from base import *
+from gi.repository import Gtk
+
+from .base import *
 class ServerFirewalled(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Server firewalled")
         page = self.initial_vbox (_("Check Server Firewall"),
                                    _("It is not possible to connect to the "
                                      "server."))
-        self.label = gtk.Label ()
+        self.label = Gtk.Label ()
         self.label.set_alignment (0, 0)
         self.label.set_line_wrap (True)
         page.pack_start (self.label, False, False, 0)
@@ -37,7 +39,7 @@ class ServerFirewalled(Question):
         if not answers['cups_queue_listed']:
             return False
 
-        if (answers.has_key ('remote_server_connect_ipp') and
+        if ('remote_server_connect_ipp' in answers and
             answers['remote_server_connect_ipp'] == False):
             self.label.set_text (_("Please check to see if a firewall or "
                                    "router configuration is blocking TCP "
