@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2010 Red Hat, Inc.
+## Copyright (C) 2010, 2014 Red Hat, Inc.
 ## Copyright (C) 2010 Jiri Popelka <jpopelka@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
@@ -19,15 +19,17 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+from gi.repository import Gtk
+
 import subprocess
-from base import *
+from .base import *
 import os
 from timedops import TimedSubprocess
 
 class VerifyPackages(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Verify packages")
-        troubleshooter.new_page (gtk.Label (), self)
+        troubleshooter.new_page (Gtk.Label (), self)
 
     def display (self):
         self.answers = {}
@@ -43,7 +45,6 @@ class VerifyPackages(Question):
                     "hpijs",
                     "hplip",
                     "system-config-printer"]
-        null = file ("/dev/null", "r+")
         parent = self.troubleshooter.get_window ()
 
         new_environ = os.environ.copy()
@@ -56,9 +57,9 @@ class VerifyPackages(Question):
                                            args=verification_args,
                                            close_fds=True,
                                            env=new_environ,
-                                           stdin=null,
+                                           stdin=subprocess.DEVNULL,
                                            stdout=subprocess.PIPE,
-                                           stderr=null)
+                                           stderr=subprocess.DEVNULL)
                 (verif_stdout, verif_stderr, result) = self.op.run ()
             except:
                 # Problem executing command.
